#!/usr/bin/env python3

import argparse
import traceback
from itertools import count
from os import makedirs, environ
from sys import argv
from typing import List, Iterable, Tuple

from vang.azdo.list_projects import list_projects
from vang.azdo.list_repos import list_repos
from vang.pio.shell import run_commands


def clone(commands, root_dir):
    makedirs(root_dir, exist_ok=True)
    yield from run_commands([(cmd, root_dir) for cmd in commands],
                            max_processes=5,
                            timeout=180,
                            check=False)


def get_commands(clone_specs, branch, flat):
    b = f' -b {branch}' if branch else ''
    return [
        f'git clone {c[0]}{b}{"" if flat else f" {c[1]}"}' for c in clone_specs
    ]


def get_clone_specs(token: str, projects: Iterable, flat: bool) -> List[Tuple]:
    return [(r['remoteUrl'],
             r['name'] if flat else f'{r["project"]["name"]}/{r["name"]}')
            for p in projects
            for r in list_repos(token, *p.split('/'))['value']]


def clone_repos(token: str, clone_dir: str, organisation: str = None, projects: List[str] = None,
                repos: List[str] = None, branch: str = None, flat: bool = True) -> List[Tuple]:
    if organisation:
        projects = list_projects(token, organisation, names=False, project_specs=True)
    elif repos:
        projects = set(['/'.join(r.split('/')[:2]) for r in repos])

    clone_specs = get_clone_specs(token, projects, flat)

    if repos:
        clone_dirs = [r.split('/')[2] if flat else '/'.join(r.split('/')[1:]) for r in repos]
        clone_specs = [(url, clone_dir) for url, clone_dir in clone_specs
                       if clone_dir in clone_dirs]

    commands = get_commands(clone_specs, branch, flat)
    for n, process in zip(count(1), clone(commands, clone_dir)):
        try:
            print(str(n).zfill(2), process.stdout.decode(), end='')
        except OSError:  # pragma: no cover
            print(traceback.format_exc())

    return clone_specs


def parse_args(args):
    parser = argparse.ArgumentParser(description='Clone Azure DevOps repos')
    parser.add_argument(
        '--token',
        default=environ.get('AZDO_TOKEN', ''),
        help='The Azure DevOps authorisation token')
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        '-o',
        '--organisation',
        help='Azure DevOps organisation, e.g organisation')
    group.add_argument(
        '-p',
        '--projects',
        nargs='+',
        help='Azure DevOps projects, e.g organisation/project')
    group.add_argument(
        '-r',
        '--repos',
        nargs='+',
        help='Repos, e.g. organisation/project/repo')
    parser.add_argument('-b', '--branch', help='The clone branch')
    parser.add_argument(
        '-d', '--clone_dir', default='.', help='The directory to clone into')
    parser.add_argument(
        '-f', '--flat', help='Clone to flat structure', action='store_true')
    return parser.parse_args(args)


def main(token: str, clone_dir: str, organisation: str, projects: List[str], repos: List[str], branch: str, flat: bool):
    for a_repo in clone_repos(
            token,
            clone_dir,
            organisation,
            projects,
            repos,
            branch,
            flat,
    ):
        print(a_repo[1])


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from os import environ
from sys import argv

from vang.jenkins.api import call


def delete_jobs(names,
                url=environ.get('JENKINS_REST_URL', None),
                username=environ.get('JENKINS_USERNAME', None),
                password=environ.get('JENKINS_PASSWORD', None),
                verify_certificate=not environ.get('JENKINS_IGNORE_CERTIFICATE', None)):
    return [(job_name, call(
        f'/job/{job_name}/doDelete',
        method='POST',
        only_response_code=True,
        rest_url=url,
        username=username,
        password=password,
        verify_certificate=verify_certificate
    )) for job_name in names]


def parse_args(args):
    parser = argparse.ArgumentParser(description='Delete Jenkins jobs')
    parser.add_argument(
        'job_names',
        nargs='+',
        help='Jenkins job names',
    )
    return parser.parse_args(args)


def main(job_names):
    for a_job_name, a_response_code in delete_jobs(job_names):
        print(a_job_name, a_response_code)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import logging
from json import loads
from os import name as os_name, system, environ
from os.path import basename
from sys import argv
from typing import Dict, Tuple

from requests import post

logging.basicConfig(level=logging.ERROR)
logger = logging.getLogger(basename(__file__))

base_url = 'https://dev.azure.com'


def create_repo(token: str, organization: str, project: str, repo: str, verify_certificate: bool = True,
                api_version: str = '6.1-preview.1') -> Tuple[int, Dict]:
    url = f'{base_url}/{organization}/{project}/_apis/git/repositories?api-version={api_version}'
    params = {
        'json': {'name': repo},
        'url': url,
        'auth': ('', token),
        'verify': verify_certificate
    }
    logger.info(f'params: {str(params).replace(token, "***")}')
    response = post(**params)
    logger.info(f'response.status_code: {response.status_code}')
    logger.info(f'response.text: {response.text}')
    response.raise_for_status()
    return response.status_code, loads(response.text)


def parse_args(args):  # pragma: no cover
    parser = argparse.ArgumentParser(
        description='Create a repo')
    parser.add_argument(
        '--token',
        default=environ.get('AZDO_TOKEN', ''),
        help='The Azure DevOps authorisation token')
    parser.add_argument(
        '--organisation',
        default=environ.get('AZDO_ORGANISATION', ''),
        help='The Azure DevOps organisation')
    parser.add_argument(
        '--project',
        default=environ.get('AZDO_PROJECT', ''),
        help='The Azure DevOps project')
    parser.add_argument(
        'repo',
        help='The Azure DevOps repo to create')
    parser.add_argument(
        '-au',
        '--azure_devops_url',
        default='https://dev.azure.com',
        help='The Azure DevOps REST API base url')

    return parser.parse_args(args)


def main(token: str, organisation: str, project: str, repo: str, azure_devops_url: str) -> None:  # pragma: no cover
    global base_url
    base_url = azure_devops_url

    status_code, repo = create_repo(token, organisation, project, repo)
    print(status_code)

    commands = f'    git remote add origin {repo["remoteUrl"]}\n' \
               '    git push -u origin 1.0.x'
    print('If you already have code ready to be pushed to this repository '
          'then run this in your terminal.')
    print(commands)
    if os_name == 'posix':
        system(f'echo "{commands}\c" | pbcopy')
        print('(The commands are copied to the clipboard)')


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from sys import argv

from vang.tfs.api import call
from vang.tfs.get_projects import get_projects


def get_repos(organisations=None,
              projects=None,
              names=False,
              repo_specs=False,
              urls=False):
    if organisations:
        projects = get_projects(organisations, project_specs=True)
    if not projects:
        return []
    repos = [(project, repo) for project in projects for repo in call(
        f'/{project}/_apis/git/repositories?api-version=3.2')['value']]
    if names:
        return [repo[1]['name'] for repo in repos]
    if repo_specs:
        return [f'{repo[0]}/{repo[1]["name"]}' for repo in repos]
    if urls:
        return [repo[1]['remoteUrl'] for repo in repos]
    return repos


def main(organisations, projects, names, repo_specs, urls):
    for repo in get_repos(organisations, projects, names, repo_specs, urls):
        print(repo)


def parse_args(args):
    parser = argparse.ArgumentParser(description='Get TFS repos')
    required_group = parser.add_mutually_exclusive_group(required=True)
    required_group.add_argument(
        '-o',
        '--organisations',
        nargs='+',
        help='TFS organisations, e.g organisation')
    required_group.add_argument(
        '-p',
        '--projects',
        nargs='+',
        help='TFS projects, e.g organisation/project')

    optional_group = parser.add_mutually_exclusive_group(required=False)
    optional_group.add_argument(
        '-n', '--names', action='store_true', help='Get only repo names')
    optional_group.add_argument(
        '-r',
        '--repo_specs',
        help='Print only organisation/project/name',
        action='store_true')
    optional_group.add_argument(
        '-u', '--urls', action='store_true', help='Get only repo urls')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.bitbucket.get_branches import get_all_branches
from vang.bitbucket.get_projects import get_projects_page
from vang.bitbucket.get_repos import get_repos_page
from vang.bitbucket.get_tags import get_all_tags
from vang.core.core import pmap_unordered


def projects(max_nb_projects=None):
    count = -1
    limit = 25
    start = 0
    is_last_page = False

    while not is_last_page:
        size, values, is_last_page, start = get_projects_page(limit, start)

        if size:
            for value in values:
                count = count + 1
                if max_nb_projects and count == max_nb_projects:
                    return value
                else:
                    yield value


def repos(project, max_nb_projects=None):
    count = -1
    limit = 25
    start = 0
    is_last_page = False

    while not is_last_page:
        size, values, is_last_page, start = get_repos_page(
            project, limit, start)

        if size:
            for value in values:
                count = count + 1
                result = (value['project']['key'], value['slug'])
                if max_nb_projects and count == max_nb_projects:
                    return result
                else:
                    yield result


def branches(repo):
    return get_all_branches(repo)


def tags(repo):
    return get_all_tags(repo)


def validate_repos(project, max_repos=None):
    valid = []
    corrupt = []
    for r in repos(project, max_repos):
        s = "/".join(r)
        try:
            branches(r)
            tags(r)
            valid.append(s)
        except KeyError:
            corrupt.append(s)
    return valid, corrupt


def validate_projects(max_projects=None, max_repos=None, max_processes=10):
    for valid, corrupt in pmap_unordered(
            lambda p: validate_repos(p['key'], max_repos),
            projects(max_projects),
            processes=max_processes):
        for v in valid:
            print(f'{v} is OK')
        for c in corrupt:
            print(f'{c} is CORRUPT')


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Validate projects by checking that the rest api can be used to retrieve repos, branches and tags.')
    parser.add_argument('-p', '--max_projects', help='Maximum number of projects', type=int, default=None)
    parser.add_argument('-r', '--max_repos', help='Maximum number of repos', type=int, default=None)
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    validate_projects(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from os import environ
from sys import argv

from requests import put


def read_file(file_path):  # pragma: no cover
    with open(file_path, 'rb') as f:
        return f.read()


def upload(file_path, repository, repository_path, url, username, password):
    url = f'{url}/repository/{repository}/{repository_path}'
    data = read_file(file_path)
    headers = {'Content-Type': 'application/octet-stream'}
    response = put(url, data=data, headers=headers, auth=(username, password))
    if response.status_code != 201:
        raise OSError(f'{response.status_code}, {response.content}')
    return response.status_code


def parse_args(args):
    parser = ArgumentParser(description='Get assets')
    parser.add_argument('file_path', help='File to upload, e.g. ./myartifact-1.0.0.jar')
    parser.add_argument('repository', help='Nexus3 repository, e.g. maven-releases')
    parser.add_argument('repository_path',
                        help='Path within Nexus3 repository, e.g com/myorg/myartifact/1.0.0/myartifact-1.0.0.jar')
    parser.add_argument('-l', '--url', default=environ.get('NEXUS3_REST_URL', None),
                        help='Nexus3 url, e.g. http://nexus_host:8080')
    parser.add_argument('-u', '--username', default=environ.get('NEXUS3_USERNAME', None), help='Nexus3 username')
    parser.add_argument('-p', '--password', default=environ.get('NEXUS3_PASSWORD', None), help='Nexus3 password')
    return parser.parse_args(args)


def main(file_path, repository, repository_path, url, username, password):
    print(upload(file_path, repository, repository_path, url, username, password))


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from os import environ, name
from os.path import exists
from sys import argv

from vang.pio.shell import run_command


def switch_settings(ending):
    source = f'{environ["HOME"]}/.m2/settings_{ending}.xml'
    if not exists(source):
        raise ValueError(f'{source} does not exist')
    target = f'{environ["HOME"]}/.m2/settings.xml'
    return run_command(f'ln -sf {source} {target}', True)


def main(ending):
    if name == 'posix':
        rc, output = switch_settings(ending)
        print(output)
    else:
        print(
            'Platform not supported. Please implement, and make a pull request.'
        )


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Switch maven settings in ~/.m2')
    parser.add_argument(
        'ending',
        help='The ending of the settings file to switch to'
        ', e.g. FOO for ~/.m2/settings_FOO.xml')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from os import environ, makedirs
from sys import argv

from requests import get

from vang.core.core import pmap_unordered


def get_default_jenkins_spec():
    return {
        'url': environ.get('JENKINS_REST_URL', None),
        'username': environ.get('JENKINS_USERNAME', None),
        'password': environ.get('JENKINS_PASSWORD', None),
        'verify_certificate': not environ.get('JENKINS_IGNORE_CERTIFICATE', None),
    }


def create_params(jenkins_spec):
    return {
        'auth': (jenkins_spec['username'], jenkins_spec['password']),
        'verify': jenkins_spec['verify_certificate']
    }


def get_config(job_name, jenkins_spec):
    params = create_params(jenkins_spec)
    params['url'] = f'{jenkins_spec["url"]}/job/{job_name}/config.xml'
    response = get(**params)
    response.raise_for_status()
    return response.status_code, response.text


def get_output_file(output_dir, job_name):
    return f'{output_dir}/{job_name}/config.xml'


def save(output_file, config):
    with open(output_file, 'wt', encoding='utf-8') as f:
        f.write(config)


def download_config(output_dir, job_name, jenkins_spec):
    try:
        makedirs(f'{output_dir}/{job_name}', exist_ok=True)
        save(get_output_file(output_dir, job_name), get_config(job_name, jenkins_spec)[1])
        return job_name
    except IOError as e:
        return f'{job_name}: {e}'


def download_configs(output_dir, job_names, jenkins_spec=get_default_jenkins_spec(), max_processes=10):
    yield from pmap_unordered(
        lambda job_name: download_config(output_dir, job_name, jenkins_spec),
        job_names,
        max_processes
    )


def get_all_job_names(jenkins_spec=get_default_jenkins_spec(), max_processes=10):
    params = create_params(jenkins_spec)
    params['url'] = f'{jenkins_spec["url"]}/api/json'
    response = get(**params)
    response.raise_for_status()
    return [job['name'] for job in response.json()['jobs']]


def parse_args(args):
    parser = argparse.ArgumentParser(description='Download Jenkins configs')
    parser.add_argument('output_dir', help='The dir to save configs in')
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('-n', '--job_names', nargs='+', help='Name of the jobs for which to download configs')
    group.add_argument('-a', '--all', action='store_true', help='Download all configs')
    return parser.parse_args(args)


def main(output_dir, job_names, all):
    for job_name in download_configs(output_dir, get_all_job_names() if all else job_names):
        print(job_name)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from shutil import copy2
from sys import argv

from vang.misc.mac_ip import get_ip_address

etc_hosts_file = '/etc/hosts'
encoding = 'utf-8'


def update(backup_file=None):
    if backup_file:
        copy2(etc_hosts_file, backup_file)

    with open(etc_hosts_file, 'rt', encoding=encoding) as f:
        original_lines = f.readlines()

    if any(['ext.local' in l for l in original_lines]):
        updated_lines = [
            f'{get_ip_address()}   ext.local\n' if 'ext.local' in l else l
            for l in original_lines
        ]
    else:
        updated_lines = list(original_lines)
        if not updated_lines[-1].endswith('\n'):
            updated_lines[-1] = updated_lines[-1] + '\n'
        updated_lines.append(f'{get_ip_address()}   ext.local\n')

    if updated_lines != original_lines:
        with open(etc_hosts_file, 'wt', encoding=encoding) as f:
            f.writelines(updated_lines)

    return updated_lines


def parse_args(args):
    parser = ArgumentParser(
        description=
        'Updates ext.local entry in /etc/hosts to the current ip address.'
        'Run it with sudo or give yourself write permission to the etc/hosts '
        'file and run it without sudo.'
        'To run it automatically, modify ext_local.plist and copy it, load and '
        'start as below: '
        'sudo cp ext_local.plist /Library/LaunchDaemons/com.github.bjuvensjo.scripts.ext_local.plist, '
        'sudo launchctl load /Library/LaunchDaemons/com.github.bjuvensjo.scripts.ext_local.plist, '
        'sudo launchctl start com.github.bjuvensjo.scripts.ext_local')
    parser.add_argument(
        '-b', '--backup', help='Backup etc/hosts in this file', default=None)
    return parser.parse_args(args)


def main(backup):
    print(''.join(update(backup)))


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from os import environ
from sys import argv
from xml.etree.ElementTree import fromstring

from requests import get


def get_metadata(root_url, username, password):
    response = get(f'{root_url}/maven-metadata.xml', auth=(username, password))
    e = fromstring(response.text)
    p = 'versioning/snapshotVersions/snapshotVersion'
    return e.findtext(f'{p}/value').strip(), e.findtext(f'{p}/extension').strip()


def download(output_file, url, username, password):
    with open(output_file, 'wb') as f:
        f.write(get(url, auth=(username, password)).content)


def main(output_dir, nexus_url, repo, group_id, artifact_id, version,
         username=environ.get('NEXUS3_USERNAME', None),
         password=environ.get('NEXUS3_PASSWORD', None)):
    root_url = f'{nexus_url}/{repo}/{group_id.replace(".", "/")}/{artifact_id}/{version}'
    snapshot_version, extension = get_metadata(root_url, username, password)
    output_file = f'{output_dir}/{artifact_id}-{version}.{extension}'
    url = f'{root_url}/{artifact_id}-{snapshot_version}.{extension}'
    print(f'Downloading: {url} to {output_file}')
    download(
        output_file,
        url,
        username,
        password
    )
    print(f'Downloaded: {url} to {output_file}')


def parse_args(args):
    parser = ArgumentParser(description='Download is_later snapshot from Nexus (requires: pipenv install requests)')
    parser.add_argument('output_dir', help='output dir')
    parser.add_argument('nexus_url',
                        help='Nexus url, e.g. http://localhost:9002/repository')
    parser.add_argument('repo', help='Nexus repository, e.g. maven-snapshots')
    parser.add_argument('group_id', help='Maven group id')
    parser.add_argument('artifact_id', help='Maven artifact id')
    parser.add_argument('version', help='Maven version')
    parser.add_argument('-u', '--username', help='username for basic auth')
    parser.add_argument('-p', '--password', help='password for basic auth')
    return parser.parse_args(args)


if __name__ == '__main__':
    main(**parse_args(argv[1:]).__dict__)



from os import environ
from pprint import pprint


def foo():
    print('foo')


def bar(x):
    if callable(x):
        x()
    else:
        print('Not callable')


bar(foo)

s = []
print(type(s))

if type(s) is str:
    print('string')
else:
    print('not string')

d = {'username': 'a', 'password': 'b', 'url': 'u'}
e = dict(d, url='new')

pprint(d)
pprint(e)

print(environ['JENKINS_USERNAME'])



#!/usr/bin/env python3
import argparse
import logging
import re
from sys import argv, stdout

from vang.pio.shell import run_command

logging.basicConfig(stream=stdout, level=logging.INFO)


def get_dependencies(dependency_tree_split):
    lines = dependency_tree_split.split('\n')
    module = lines[0].replace('[INFO]', '').strip()
    direct = []
    transitive = []
    for line in lines[1:]:
        if line.startswith('[INFO] +- ') or line.startswith('[INFO] \- '):
            direct.append(line.split('-', maxsplit=1)[1].strip())
        else:
            transitive.append(line.split('-', maxsplit=1)[1].strip())
    return module, sorted(direct), sorted(transitive)


def split_dependency_tree(dependency_tree):
    """Splits a multi module into separate parts and a single module to one."""
    splits = re.split(r'\[INFO\] --- maven-dependency-plugin.*', dependency_tree)
    if len(splits) < 2:
        return []
    modules = []
    for m in splits[1:]:
        lines = []
        for line in m.strip().split('\n'):
            if len(line.split(':')) >= 4:
                lines.append(line)
            else:
                break
        modules.append('\n'.join(lines))

    return modules


def get_dependency_tree(pom_file):
    code, output = run_command(f'mvn dependency:tree -f {pom_file}', True)
    return output


def get_modules_dependencies(pom_file):
    dependency_tree = get_dependency_tree(pom_file)
    logging.debug('dependency tree: %s', dependency_tree)
    modules_dependency_trees = split_dependency_tree(dependency_tree)
    logging.debug('modules dependency tree: %s', modules_dependency_trees)
    modules_dependencies = [get_dependencies(mdt) for mdt in modules_dependency_trees]
    return modules_dependencies


def main(pom_file, only_direct, only_transitive):
    for module, direct, transitive in get_modules_dependencies(pom_file):
        print('#' * 80)
        print(module)
        if not only_transitive:
            print('direct:\n  ', '\n  '.join(direct), sep='')
        if not only_direct:
            print('transitive:\n  ', '\n  '.join(transitive), sep='')
        print('#' * 80)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Get direct and transitive dependencies from provided pom.xml, '
                    'e.g. ./get_dependencies.py ./pom.xml. '
                    'Depends on Maven.')
    parser.add_argument('-p', '--pom_file', help='Path to pom file', default='pom.xml')
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        '-d', '--only_direct', help='Print only direct dependants', action='store_true')
    group.add_argument(
        '-t', '--only_transitive', help='Print only transitive dependants', action='store_true')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from glob import glob
from os import makedirs
from os.path import exists
from re import match
from sys import argv

from vang.pio.shell import run_command


def is_excluded(py_file):
    return any((match(regexp,
                      py_file.split('/')[-1])
                for regexp in ['__init__.py', '.*slask.*', '.*Slask.*']))


def has_main(py_file):
    main_lines = [
        "if __name__ == '__main__':",
        'if __name__ == "__main__":',
    ]
    with open(py_file, 'rt', encoding='utf-8') as f:
        for line in f:
            # if line and match(r'.*if __name__ == ("|\')__main__("|\'): *',
            #                   line):
            print(line)
            if line.strip() in main_lines:
                return True
    return False


def map_to_link_name(main_file):
    main_file_package, main_file_name = main_file.split('/')[-2:]
    return f'{main_file_package}-{main_file_name[:-3].replace("_", "-")}'


def create_symlinks(source, target):
    if not exists(target):
        makedirs(target)

    for main_file in [
            f for f in glob(f'{source}/**/*.py', recursive=True)
            if has_main(f) and not is_excluded(f)
    ]:
        link_file = f'{target}/{map_to_link_name(main_file)}'
        if exists(link_file):
            print(f'{link_file} already exists')
        else:
            command = f'ln -s {main_file} {link_file}'
            print(command)
            run_command(command)


def parse_args(args):
    parser = ArgumentParser(description='Create symlinks')
    parser.add_argument('source', help='Source root')
    parser.add_argument('target', help='Target')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    create_symlinks(**parse_args(argv[1:]).__dict__)




from vang.artifactory.mirror import not_folders, not_index, not_maven_metadata, latest_version

work_dir = './work_dir'
# Spec of the artifactory instance to copy from
from_artifactory_spec = {
    'url': '<from_artifactory_url>',
    'username': '<username>',
    'password': '<password>',
}
# Spec of the artifactory instance to copy to
# In example below, by creating a copy of from_artifactory_spec with a different url
to_artifactory_spec = dict(from_artifactory_spec, url='<to_artifactory_url>')

# Specify a pair (tuple) if the repo key to mirror to is not the same as the repo key to mirror from
repositories = [
    ('from_repo_key', 'to_repo_key'),
    'from_and_to_repo_key'
]

filters = (not_folders, not_index, not_maven_metadata, latest_version)



#!/usr/bin/env python3
from argparse import ArgumentParser
from os import environ, path, walk
from os.path import normpath, exists
from sys import argv
from xml.etree.ElementTree import parse

NAME_SPACES = {'ns': 'http://maven.apache.org/SETTINGS/1.0.0'}


def text(e, n):
    return e.findtext(f'ns:{n}', namespaces=NAME_SPACES)


def findall(e, n):
    return e.findall(f'.//ns:{n}', namespaces=NAME_SPACES)


def get_settings_info(settings_path):
    e = parse(normpath(settings_path))

    local_repo = text(e, 'localRepository')
    if '${user.home}' in local_repo:
        local_repo = local_repo.replace('${user.home}', environ['HOME'])

    remotes = {text(r, 'id'): text(r, 'url')
               for r in findall(e, 'repository')}

    plugin_remotes = {text(r, 'id'): text(r, 'url')
                      for r in findall(e, 'pluginRepository')}

    mirrors = {text(r, 'id'): text(r, 'url')
               for r in findall(e, 'mirror')}

    return local_repo, {**remotes, **plugin_remotes, **mirrors}


def excluded(f):
    if f in ['_remote.repositories',
             'maven-metadata-local.xml',
             'maven-metadata-ossrh.xml',
             'resolver-status.properties']:
        return True
    if f.endswith('asc'):
        return True
    if f.endswith('lastUpdated'):
        return True
    return False


def get_file_repos_info(local_repo, files):
    for f in files:
        root, file = f.rsplit('/', 1)
        remote_repositories = path.join(root, '_remote.repositories')
        if exists(remote_repositories):
            yield (remote_repositories,
                   root[len(local_repo):],
                   (file,))


def get_repos_info(local_repo):
    for root, dirs, files in walk(local_repo):
        if files and '_remote.repositories' in files:
            yield (path.join(root, '_remote.repositories'),
                   root[len(local_repo):],
                   [f for f in files
                    if not excluded(f)])


def parse_remote_repos(remote_repositories, remotes):
    with open(remote_repositories, 'rt', encoding='utf-8') as f:
        lines = [l for l in f.read().splitlines() if not l.startswith('#')]
        return {k: remotes.get(v.replace('=', ''), None) for k, v in
                [l.split('>') for l in lines]}


def create_commands(remote_dict, artifact_path, artifacts):
    def create_artifact_command(artifact):
        extension = artifact.rsplit('.', 1)[1]
        remote_dict_key = artifact[:-(len(extension) + 1)] if extension in [
            'sha1', 'md5', 'sha256'] else artifact

        remote_url = remote_dict.get(remote_dict_key, None)
        if remote_url:
            if remote_url.endswith('/'):
                remote_url = remote_url[:-1]
            rel_path = f'{artifact_path}/{artifact}'
            return f'curl -o "{rel_path}" {remote_url}/{rel_path}'
        else:
            return None

    mkdir_commands = [f'mkdir -p "{artifact_path}"']
    download_commands = [create_artifact_command(artifact) for artifact in
                         artifacts]
    return mkdir_commands + download_commands if any(download_commands) else []


def create_urls(remote_dict, artifact_path, artifacts):
    def create_artifact_urls(artifact):
        extension = artifact.rsplit('.', 1)[1]
        remote_dict_key = artifact[:-(len(extension) + 1)] if extension in [
            'sha1', 'md5', 'sha256'] else artifact

        remote_url = remote_dict.get(remote_dict_key, None)
        if remote_url:
            if remote_url.endswith('/'):
                remote_url = remote_url[:-1]
            rel_path = f'{artifact_path}/{artifact}'
            return f'{remote_url}/{rel_path}'
        else:
            return f'No url found for: {artifact_path}/{artifact}'

    urls = [create_artifact_urls(artifact) for artifact in artifacts]
    return urls if any(urls) else []


def main(settings_file, urls_only, skipped, files=None):
    skipped_artifacts = []
    local_repo, remotes = get_settings_info(settings_file)

    repos_info = get_file_repos_info(local_repo, files) if files else get_repos_info(local_repo)
    for remote_repos, artifact_path, artifacts in repos_info:
        remote_dict = parse_remote_repos(remote_repos, remotes)

        params = [remote_dict, artifact_path, artifacts]
        outputs = create_urls(*params) if urls_only else create_commands(
            *params)

        if outputs:
            for c in outputs:
                print(c)
        else:
            skipped_artifacts.append(artifact_path)

    if skipped:
        for s in skipped_artifacts:
            print('# Skipped:', s)


def parse_args(args):
    parser = ArgumentParser(
        description='Print download commands (POSIX) to enable making '
                    'a remote repo based on the content of a local')
    parser.add_argument(
        'settings_file', help='Settings file')
    parser.add_argument(
        '-u', '--urls_only', help='Print only urls', action='store_true')
    parser.add_argument(
        '-s', '--skipped', help='Print skipped artifacts', action='store_true')
    parser.add_argument(
        '-f', '--files', nargs='*', help='Only for specified files and not all in local_repo')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from json import dumps
from sys import argv

from vang.tfs.api import call_url
from vang.tfs.list_release_definitions import list_release_definitions


def get_release_definitions(organisations=None,
                            projects=None,
                            filter_name=None, ):
    release_definitions_dict = list_release_definitions(organisations, projects, filter_name)
    return {name: call_url(definition['url'])
            for name, definition in release_definitions_dict.items()}


def main(organisations,
         projects,
         filter_name=None,
         format_output=False):
    for name, definition in get_release_definitions(organisations, projects, filter_name).items():
        json = dumps(definition, indent=4) if format_output else dumps(definition)
        print(json)


def parse_args(args):
    parser = argparse.ArgumentParser(description='Get TFS release definitions')
    required_group = parser.add_mutually_exclusive_group(required=True)
    required_group.add_argument(
        '-o',
        '--organisations',
        nargs='+',
        help='TFS organisations, e.g organisation')
    required_group.add_argument(
        '-p',
        '--projects',
        nargs='+',
        help='TFS projects, e.g organisation/project')

    parser.add_argument(
        '-fn',
        '--filter_name',
        help='Filters to definitions whose names equal this value. Append a * to filter to definitions whose names '
             'start with this value')

    optional_group = parser.add_mutually_exclusive_group(required=False)
    optional_group.add_argument(
        '-f', '--format_output', action='store_true',
        help='Format output')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from os import name as os_name
from os import system
from sys import argv

from vang.tfs.api import call


def create_repo(repo):
    organisation, project, name = repo.split('/')
    return call(
        f'/{organisation}/{project}/_apis/git/repositories?api-version=3.2',
        request_data={
            'name': name
        },
        method='POST',
    )


def main(repo):
    response = create_repo(repo)
    commands = f'    git remote add origin {response["remoteUrl"]}\n' \
        '    git push -u origin develop'
    print('If you already have code ready to be pushed to this repository '
          'then run this in your terminal.')
    print(commands)
    if os_name == 'posix':
        system(f'echo "{commands}\c" | pbcopy')
        print('(The commands are copied to the clipboard)')


def parse_args(args):
    parser = argparse.ArgumentParser(description='Create TFS repository')
    parser.add_argument(
        'repo',
        help='The TFS repository to create, e.g. organisation/project/repo1')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import hashlib
from itertools import count
from json import dumps
from os import makedirs
from os.path import exists

from requests import get, put

import vang.artifactory.mirror_config as config
from vang.artifactory.diff_repos import diff, get_repo_content
from vang.core.core import pmap_unordered
from vang.misc.version import get_latest_snapshot, get_latest_rc, get_latest_fixed


def read_file(file_path):  # pragma: no cover
    with open(file_path, 'rb') as f:
        return f.read()


def get_checksums(the_bytes):
    return (
        hashlib.md5(the_bytes).hexdigest(),
        hashlib.sha1(the_bytes).hexdigest(),
        hashlib.sha256(the_bytes).hexdigest()
    )


def sha1sum(file_path):
    return get_checksums(read_file(file_path))[1]


def download_file(output_file, url, sha1, retry=True):
    if exists(output_file) and sha1sum(output_file) == sha1:
        return True, 'Already downloaded'
    with open(output_file, 'wb') as f:
        result = get(url)
        if result.status_code == 200:
            f.write(result.content)
            if sha1sum(output_file) == sha1:
                return True, 'Downloaded and verified sha1'
            else:
                if retry:
                    return download_file(output_file, url, sha1, False)
                else:
                    return False, 'Downloaded with bad sha1'
        else:
            if retry:
                return download_file(output_file, url, sha1, False)
            else:
                return False, f'{result.status_code}: {result.content}'


def get_checksum_headers(md5, sha1, sha256):
    return {
        'X-Checksum-Md5': md5,
        'X-Checksum-Sha1': sha1,
        'X-Checksum-Sha256': sha256
    }


def mirror_file(repo_work_dir, from_repo_uri, artifact_spec, to_repo_uri, to_repo_username,
                to_repo_password):
    state = {
        'download_url': f'{from_repo_uri}{artifact_spec["uri"]}',
    }
    # download
    d, f = artifact_spec['uri'].rsplit('/', maxsplit=1)
    output_dir = f'{repo_work_dir}{d}'
    makedirs(output_dir, exist_ok=True)
    state['download_output_file'] = f'{repo_work_dir}{artifact_spec["uri"]}'
    download_result, download_message = download_file(state['download_output_file'], state['download_url'],
                                                      artifact_spec['sha1'],
                                                      True)
    # TODO Return checksums from download and use in upload
    state['download_result'] = download_result
    state['download_message'] = download_message

    # upload
    if download_result:
        content = read_file(state['download_output_file'])
        md5, sha1, sha256 = get_checksums(content)
        state['headers'] = {
            **get_checksum_headers(md5, sha1, sha256),
            'Content-Type': 'application/octet-stream'
        }
        state['upload_url'] = f'{to_repo_uri}{artifact_spec["uri"]}'
        upload_result = put(state['upload_url'], data=content, headers=state['headers'],
                            auth=(to_repo_username, to_repo_password))
        state['upload_result'] = upload_result.status_code == 201
        state['upload_status_code'] = upload_result.status_code
        state['upload_response'] = upload_result.text
        state['result'] = state['upload_result']

    return state


def not_folders(repo_files):
    return filter(lambda r: not r['folder'], repo_files)


def not_maven_metadata(repo_files):
    return filter(lambda r: not r['uri'].endswith('maven-metadata.xml'), repo_files)


def not_index(repo_files):
    return filter(lambda r: '.index' not in r['uri'], repo_files)


def latest_version(repo_files):
    version_map = {}
    for repo_file in repo_files:
        uri = repo_file['uri']
        group_and_artifact_id = '/'.join(uri.split('/')[:-2])
        version = uri.split('/')[-2]
        if group_and_artifact_id not in version_map:
            version_map[group_and_artifact_id] = []
        version_map[group_and_artifact_id].append(version)

    latest_version_map = {}
    for k, v in version_map.items():
        try:
            latest_version_map[k] = [get_latest_snapshot(v), get_latest_rc(v), get_latest_fixed(v)]
        except ValueError:
            latest_version_map[k] = v

    latest_repo_files = []
    for repo_file in repo_files:
        uri = repo_file['uri']
        group_and_artifact_id = '/'.join(uri.split('/')[:-2])
        version = uri.split('/')[-2]
        if version in latest_version_map[group_and_artifact_id]:
            latest_repo_files.append(repo_file)

    return latest_repo_files


def filter_repo_files(repo_files, filters):
    print(f'all: {len(repo_files)}')
    for f in filters:
        repo_files = f(repo_files)
        repo_files = list(repo_files)
        print(f'{f.__name__}: {len(repo_files)}')
    return repo_files


# TODO Maybe use "Last downloaded" to filter out even more
# Note that files ending SNAPSHOT.* in from repo will get SNAPSHOT replaced
# by a timestamp in to repo and thus will not be found in to_uri
# on a rerun of this script. For now, nothing will be done about that.
def mirror(filters, max_processes=10):
    for repo_key in config.repositories:
        if type(repo_key) is str:
            from_repo_key = repo_key
            to_repo_key = repo_key
        else:
            from_repo_key, to_repo_key = repo_key

        from_repo_files = get_repo_content(from_repo_key, **config.from_artifactory_spec)['files']
        filtered_from_repo_files = filter_repo_files(from_repo_files, filters)

        to_repo_files = get_repo_content(to_repo_key, **config.to_artifactory_spec)['files']
        files_not_in_to_repo, files_not_in_from_repo = diff(filtered_from_repo_files, to_repo_files, only_keys=False)
        files_not_in_to_repo = list(files_not_in_to_repo)
        print('Not in to repo:', len(files_not_in_to_repo))

        from_repo_uri = f'{config.from_artifactory_spec["url"]}/{from_repo_key}'
        to_repo_uri = f'{config.to_artifactory_spec["url"]}/{to_repo_key}'
        yield from pmap_unordered(
            lambda artifact_spec: mirror_file(f'{config.work_dir}/{from_repo_key}', from_repo_uri, artifact_spec,
                                              to_repo_uri, config.to_artifactory_spec['username'],
                                              config.to_artifactory_spec['password']),
            files_not_in_to_repo,
            processes=max_processes)


if __name__ == '__main__':
    verbose = True
    for n, m in zip(count(), mirror(config.filters)):
        if verbose or not m['result']:
            print(n, dumps(m))
    print('Done!')



#!/usr/bin/env python3
import argparse
import itertools
from sys import argv

from vang.bitbucket.api import get_all
from vang.core.core import pmap_unordered


def get_repos(project, only_name=False, only_spec=False):
    for value in get_all(f'/rest/api/1.0/projects/{project}/repos'):
        if only_name:
            yield value['slug']
        elif only_spec:
            yield value['project']['key'], value['slug']
        else:
            yield value


def get_all_repos(projects, max_processes=10, only_name=False, only_spec=False):
    return itertools.chain.from_iterable(
        pmap_unordered(
            lambda p: get_repos(p, only_name, only_spec),
            projects,
            processes=max_processes))


def main(projects, name, repo_specs):
    for repo in get_all_repos(projects, only_name=name, only_spec=repo_specs):
        if repo_specs:
            print(f'{repo[0]}/{repo[1]}')
        else:
            print(repo)


def parse_args(args):
    parser = argparse.ArgumentParser(description='Get repos from Bitbucket')
    parser.add_argument('projects', nargs='+', help='Project keys')
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-n', '--name', help='Print only repo name', action='store_true')
    group.add_argument(
        '-r',
        '--repo_specs',
        help='Print only project_key/name',
        action='store_true')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from vang.jenkins.copy_job import copy_job


def copy_jobs(jobs, from_jenkins_spec, to_jenkins_spec, replacements=[], verbose=False):
    for job in jobs:
        from_name = None
        to_name = None
        job_replacements = replacements
        try:
            if type(job) is str:
                to_name = from_name = job
            elif type(jobs) is list and len(job) == 2:
                from_name, to_name = job
            elif type(jobs) is list and len(job) == 3:
                from_name, to_name, job_replacements = job

            if type(from_name) != str or type(to_name) != str or not (
                    type(job_replacements) is list or callable(job_replacements)):
                raise ValueError('job must be a string, a pair of strings or a pair of string and a callable')

            response = copy_job(from_name, to_name, from_jenkins_spec, to_jenkins_spec, job_replacements)
            response_code = response['response_code']
            print('Created copy' if response_code == 200 else 'Failed to create copy', to_name, response_code)
            if verbose:
                print(response)
        except IOError as r:
            print('IOError while trying to create copy', to_name, r)
        except ValueError as v:
            print('ValueError while trying to create copy', to_name, v)



#!/usr/bin/env python3

from os import environ

from requests import delete, get, post, put


def call(
        uri,
        request_data=None,
        method='GET',
        only_response_code=False,
        rest_url=environ.get('JENKINS_REST_URL', None),
        username=environ.get('JENKINS_USERNAME', None),
        password=environ.get('JENKINS_PASSWORD', None),
        verify_certificate=not environ.get('JENKINS_IGNORE_CERTIFICATE', None),
):
    """Makes a REST call to Jenkins rest api.
    May use three environment variables:
    * JENKINS_REST_URL, e.g. http://myorg.com/stash
    * JENKINS_USERNAME, the jenkins username
    * JENKINS_PASSWORD, the jenkins password

    Args:
        uri (str): e.g. "/api/json"
        request_data (dict): the JSON request
        method: http method
        only_response_code: default False
        rest_url: default environ.get('JENKINS_REST_URL', None)
        username: default environ.get('JENKINS_USERNAME', None)
        password: default environ.get('JENKINS_PASSWORD', None)
        verify_certificate: True if https certificate should be verified

    Return:
          the response
    """
    return call_url(f'{rest_url}{uri}', request_data, method, only_response_code, username, password,
                    verify_certificate)


def call_url(
        url,
        request_data=None,
        method='GET',
        only_response_code=False,
        username=environ.get('JENKINS_USERNAME', None),
        password=environ.get('JENKINS_PASSWORD', None),
        verify_certificate=True,
):
    m = {'DELETE': delete,
         'GET': get,
         'POST': post,
         'PUT': put,
         }[method]

    params = {'url': url, 'auth': (username, password), 'verify': verify_certificate}
    if request_data:
        params['json'] = request_data

    response = m(**params)
    return response.status_code if only_response_code else response.json() if response.text else response.status_code()



#!/usr/bin/env python3
""" Maven pom utilities. """

from glob import glob
from os.path import realpath, normpath
from xml.etree.ElementTree import parse

NAME_SPACES = {'ns': 'http://maven.apache.org/POM/4.0.0'}


def get_pom_paths(root):
    """ Returns the absolute path of poms that recursively are found in root"""
    return [realpath(p) for p in glob(f"{root}/**/pom.xml", recursive=True)]


def get_pom_info(pom_path):
    """ Returns a dictionary with pom_path, artifact_id,
    group_id, version and packaging. """
    e = parse(normpath(pom_path))
    info = {
        'pom_path': pom_path,
        'artifact_id': e.findtext('ns:artifactId', namespaces=NAME_SPACES) or e.findtext('artifactId'),
        'group_id': e.findtext('ns:groupId', namespaces=NAME_SPACES) or e.findtext('groupId'),
        'version': e.findtext('ns:version', namespaces=NAME_SPACES)  or e.findtext('version'),
        'packaging': e.findtext('ns:packaging', namespaces=NAME_SPACES) or e.findtext('packaging') or 'jar'
    }
    if not info['group_id']:
        info['group_id'] = e.findtext('ns:parent/ns:groupId', namespaces=NAME_SPACES) or e.findtext('parent/groupId')
    if not info['version']:
        info['version'] = e.findtext('ns:parent/ns:version', namespaces=NAME_SPACES) or e.findtext('parent/version')

    return info



#!/usr/bin/env python3

from os import name, system

from vang.maven.pom import get_pom_info


def get_artifact_id(pom_path):
    return get_pom_info(pom_path)['artifact_id']


def main():
    artifact_id = get_artifact_id('./pom.xml')
    if name == 'posix':
        system(f'echo "{artifact_id}\c" | pbcopy')
        print(f'"{artifact_id}" copied to clipboard')
    else:
        print(artifact_id)


if __name__ == '__main__':  # pragma: no cover
    main()



#!/usr/bin/env python3
from argparse import ArgumentParser
from sys import argv

from vang.bitbucket.create_repo import create_repo
from vang.bitbucket.enable_webhooks import enable_repo_web_hook
from vang.bitbucket.get_clone_urls import get_clone_urls
from vang.bitbucket.set_default_branches import set_repo_default_branch
from vang.misc.s import get_zipped_cases
from vang.pio.rsr import get_replace_function, rsr
from vang.pio.shell import run_command


def setup(project, repo, src_branch, dest_project, dest_repo, dest_branch, work_dir):
    clone_url = [c for _, p, r, c in get_clone_urls([project]) if r == repo][0]
    dest_repo_dir = f'{work_dir}/{dest_project}/{dest_repo}'
    run_command(f'git clone -b {src_branch} {clone_url} {dest_repo_dir}')
    run_command('rm -rf .git', return_output=True, cwd=dest_repo_dir)
    run_command('git init', return_output=True, cwd=dest_repo_dir)
    if not dest_branch == 'master':
        run_command(f'git checkout -b {dest_branch}', return_output=True, cwd=dest_repo_dir)
    return clone_url, dest_repo_dir


def commit_all(repo_dir):
    run_command('git add --all', return_output=True, cwd=repo_dir)
    run_command(
        'git commit -m "Initial commit"',
        return_output=True,
        cwd=repo_dir,
    )


def update(repo, dest_repo, dest_repo_dir):
    for old, new in get_zipped_cases([repo, dest_repo]):
        rsr(old, new, [dest_repo_dir], get_replace_function(False))


def create_and_push_to_dest_repo(dest_project,
                                 dest_repo,
                                 dest_branch,
                                 dest_repo_dir,
                                 webhook=None):
    dest_repo_origin = create_repo(dest_project, dest_repo)['links']['clone'][0]['href']
    if webhook:
        enable_repo_web_hook([dest_project, dest_repo], webhook)
    run_command(f'git remote add origin {dest_repo_origin}',
                return_output=True,
                cwd=dest_repo_dir)
    run_command(f'git push -u origin {dest_branch}', return_output=True, cwd=dest_repo_dir)
    set_repo_default_branch((dest_project, dest_repo), dest_branch)
    return dest_repo_origin


def parse_args(args):
    parser = ArgumentParser(
        description='Create a new repo based on template repo.\nExample: '
                    'create_from_template PCS1806 foo PCS1806 bar -sb '
                    'develop -db 1.0.x -d . -w http://my-host:7777/wildcat/webhook/')
    parser.add_argument(
        'src_project',
        help='The project from which to create the new repo (must exist)')
    parser.add_argument(
        'src_repo',
        help='The repo from which to create the new repo (must exist)')
    parser.add_argument(
        'dest_project',
        help='The project in which to create the new repo (must exist)')
    parser.add_argument(
        'dest_repo',
        help='The new repo (must not exist)')
    parser.add_argument(
        '-sb',
        '--src_branch',
        default='develop',
        help='The branch to use, e.g. develop')
    parser.add_argument(
        '-db',
        '--dest_branch',
        help='The branch to create, e.g. develop. '
             'Will be set to src_branch if not specified. '
             'It will be set as default branch on created repo')
    parser.add_argument(
        '-d',
        '--work_dir',
        default='.',
        help='The directory to create local repo in (directory work_dir/src_project/src_repo must not exists)')
    parser.add_argument(
        '-w',
        '--webhook',
        default=False,
        help='Webhook url to add to the new repo')
    return parser.parse_args(args)


def main(src_project, src_repo, src_branch, dest_project, dest_repo, dest_branch, work_dir, webhook):
    if not dest_branch:
        dest_branch = src_branch
    clone_url, dest_repo_dir = setup(src_project, src_repo, src_branch, dest_project, dest_repo, dest_branch, work_dir)
    update(src_repo, dest_repo, dest_repo_dir)
    commit_all(dest_repo_dir)
    dest_repo_origin = create_and_push_to_dest_repo(dest_project, dest_repo, dest_branch, dest_repo_dir, webhook)
    print('Created', dest_repo_origin)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from os import environ
from sys import argv

from requests import post

from vang.jenkins.download_configs import get_config


def get_default_jenkins_spec():
    return {
        'url': environ.get('JENKINS_REST_URL', None),
        'username': environ.get('JENKINS_USERNAME', None),
        'password': environ.get('JENKINS_PASSWORD', None),
        'verify_certificate': not environ.get('JENKINS_IGNORE_CERTIFICATE', None),
    }


def create_params(jenkins_spec):
    return {
        'auth': (jenkins_spec['username'], jenkins_spec['password']),
        'verify': jenkins_spec['verify_certificate']
    }


def create_job(jenkins_spec, name, config):
    params = create_params(jenkins_spec)
    params['url'] = f'{jenkins_spec["url"]}/createItem?name={name}'
    params['data'] = config
    params['headers'] = {'Content-Type': 'application/xml'}
    response = post(**params)
    return response.status_code


def update_config(config, replacements):
    if callable(replacements):
        return replacements(config)
    else:
        for r in replacements:
            config = config.replace(*r)
        return config


def copy_job(from_name, to_name, from_jenkins_spec=get_default_jenkins_spec(),
             to_jenkins_spec=get_default_jenkins_spec(), replacements=None):
    from_response_code, from_config = get_config(from_name, from_jenkins_spec)
    to_config = update_config(from_config, replacements)
    to_response_code = create_job(to_jenkins_spec, to_name, to_config)
    return {
        'from_name': from_name,
        'to_name': to_name,
        'from_config': from_config,
        'to_config': to_config,
        'response_code': to_response_code,
    }


def parse_args(args):
    parser = argparse.ArgumentParser(description='Copy Jenkins job')
    parser.add_argument('from_name', help='Name of job to copy')
    parser.add_argument('to_name', help='Name of job copy')
    parser.add_argument('-r', '--replacements', nargs='*', default=[],
                        help='Pair of replacements, e.g. old1 new1 old2 new2')
    return parser.parse_args(args)


def main(from_name, to_name, replacements=()):
    response = copy_job(from_name, to_name, replacements=zip(replacements[0::2], replacements[1::2]))
    response_code = response['response_code']
    print('Created' if response_code == 200 else 'Failed to create', to_name, response_code)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.tfs.api import call_url
from vang.tfs.definition_utils import get_definition, get_definition_name
from vang.tfs.list_release_definitions import list_release_definitions


def get_current_definition(organisation, project, definition_name):
    return list_release_definitions(projects=(f'{organisation}/{project}',), filter_name=definition_name)[
        definition_name]


def get_release_definition(template, organisation, project, repo, branch, definition_name, definition_id, revision,
                           comment=None):
    additions = {'id': definition_id, 'revision': revision}
    if comment:
        additions['comment'] = comment
    return get_definition(template,
                          {
                              'name': definition_name,
                              'organisation': organisation,
                              'project': project,
                              'repo': repo,
                              'branch': branch
                          },
                          additions)


def update_release_definition(definition_url, release_definition):
    return call_url(
        f'{definition_url}?api-version=3.2-preview',
        request_data=release_definition,
        method='PUT',
        only_response_code=True
    )


def main(project, repo, branch, template, comment=None):
    organisation, project = project.split('/')
    definition_name = get_definition_name(project, repo, branch)
    definition = get_current_definition(organisation, project, definition_name)
    definition_url = definition['url']
    definition_id = definition['id']
    revision = definition['revision']
    response = update_release_definition(definition_url,
                                         get_release_definition(template, organisation, project, repo,
                                                                branch, definition_name, definition_id, revision,
                                                                comment))
    print(response)


def parse_args(args):
    parser = argparse.ArgumentParser(description='Update TFS release definitions')
    parser.add_argument(
        'project',
        help='TFS projects, e.g organisation/project')
    parser.add_argument(
        'repo',
        help='The TFS git repo name, e.g. spam.eggs')
    parser.add_argument(
        'branch',
        help='The TFS git repo branch, e.g. 1.0.x')
    parser.add_argument(
        'template',
        help='The TFS release definition template file, e.g. release_definition_template.json')
    parser.add_argument(
        '-c',
        '--comment',
        help='The comment to add to the release definition, e.g. a commit sha of the release definition template')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import sys
from itertools import chain

from vang.bitbucket.api import get_all
from vang.bitbucket.utils import get_repo_specs
from vang.core.core import pmap_unordered


# This is not used in this script but still a useful function :)
def get_branch_commits(repo_specs_and_branches, take=sys.maxsize, max_processes=10, filter_function=None):
    def inner(rsb):
        spec = rsb[0]
        branches = rsb[1]
        result = {}
        for b in branches:
            commits = get_all(f'/rest/api/1.0/projects/{spec[0]}/repos/{spec[1]}/commits',
                              {'until': b},
                              take)
            result[b] = filter_function(commits) if filter_function else commits

        return result

    return pmap_unordered(
        lambda r: {r[0]: inner(r)},
        repo_specs_and_branches,
        processes=max_processes)


def get_commits(repo_specs, branches=tuple(['']), take=sys.maxsize, max_processes=10):
    return pmap_unordered(
        lambda s: list([s, chain([(b, get_all(f'/rest/api/1.0/projects/{s[0]}/repos/{s[1]}/commits',
                                              {'until': b},
                                              take))
                                  for b in branches])]),
        repo_specs,
        processes=max_processes)


def main(take, branches, sha=False, dirs=None, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, commits in get_commits(specs, branches, take=take):
        if sha:
            for b, cs in commits:
                for c in cs:
                    # print(b, c['id'])
                    print(c['id'])
        else:
            print(f'{spec[0]}/{spec[1]}: {list(commits)}')


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Get repository commits from Bitbucket')
    parser.add_argument('-b', '--branches', nargs='*', help='Branches filter', default=('',))
    parser.add_argument('--take', '-t', type=int,
                        help='The number of commit from and including HEAD. -1 for all commits',
                        default=sys.maxsize)
    parser.add_argument(
        '-s', '--sha', help='Print only sha', action='store_true')
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-d',
        '--dirs',
        nargs='*',
        default=['.'],
        help='Git directories to extract commit information from')
    group.add_argument(
        '-r', '--repos', nargs='*', help='Repos, e.g. key1/repo1 key2/repo2')
    group.add_argument(
        '-p', '--projects', nargs='*', help='Projects, e.g. key1 key2')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(sys.argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import sys
from functools import partial
from multiprocessing.dummy import Pool
from traceback import print_exc

import vang.bitbucket.clone_repos as clone_repos
from vang.bitbucket.create_repo import create_repo
from vang.bitbucket.set_default_branches import set_repo_default_branch
from vang.pio.command_all import get_work_dirs
from vang.pio.shell import run_command


def fork_repo(fork_project, branches, git_dir):
    try:
        for b in reversed(branches):
            run_command(f'git checkout {b}', return_output=True, cwd=git_dir)
        rc, original_origin = run_command(
            'git remote get-url origin', return_output=True, cwd=git_dir)
        repo = original_origin.split('/')[-1][:-4]
        new_origin = '/'.join(original_origin.split(
            '/')[:-2]) + '/' + fork_project.lower() + '/' + repo + '.git'
        create_repo(fork_project, repo)
        run_command(
            f'git remote set-url origin {new_origin}',
            return_output=True,
            cwd=git_dir)
        run_command('git remote prune origin', return_output=True, cwd=git_dir)
        for b in branches:
            run_command(
                f'git push -u origin {b}', return_output=True, cwd=git_dir)
        set_repo_default_branch((fork_project, repo), branches[0])

        print(f'{fork_project}/{repo}: {new_origin}')
        return (fork_project, repo), new_origin
    except OSError:  # pragma: no cover
        print(f'Error forking {git_dir}')
        print_exc(file=sys.stdout)


def fork_repos(fork_project,
               branches,
               work_dir='.',
               repos=None,
               projects=None,
               max_processes=10):
    clone_repos.main(work_dir, projects, repos, branch=branches[0], flat=True)
    git_dirs = get_work_dirs('.git/', work_dir)
    with Pool(processes=max_processes) as pool:
        pool.map(partial(fork_repo, fork_project, branches), git_dirs)


def main(fork_project, branches, work_dir='.', repos=None, projects=None):
    fork_repos(fork_project, branches, work_dir, repos, projects)


def parse_args(args):
    parser = argparse.ArgumentParser(description='''
       Fork Bitbucket repos by cloning repos,
       create new repos in fork project,
       set them as origin
       and push branches to new repos
    ''')
    parser.add_argument('fork_project', help='Fork project (must exist)')
    parser.add_argument(
        '-b',
        '--branches',
        nargs='*',
        help='The branches to fork, e.g. develop master. '
        'The first will be set as default branch',
        default=['develop'])
    parser.add_argument(
        '-d', '--work_dir', default='.', help='The directory to clone into. Preferably an empty directory!')
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        '-r', '--repos', nargs='*', help='Repos, e.g. key1/repo1 key2/repo2')
    group.add_argument(
        '-p', '--projects', nargs='*', help='Projects, e.g. key1 key2')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(sys.argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import logging
from os import environ
from os.path import basename
from sys import argv
from typing import Iterable

from requests import patch

from vang.azdo.list_builds import list_builds

logging.basicConfig(level=logging.ERROR)
logger = logging.getLogger(basename(__file__))

base_url = 'https://dev.azure.com'


def cancel_build(build_id: str, token: str, organisation: str, project: str, verify_certificate: bool = True,
                 api_version: str = '6.1-preview.6') -> Iterable:
    ids = []
    builds = list_builds(token, organisation, project, verify_certificate)
    for b in builds:
        if b['status'] != 'completed':
            print(b['status'])
            b['status'] = 'Cancelling'
            build_id = b['id']

            url = f'{base_url}/{organisation}/{project}/_apis/build/builds/{build_id}?api-version={api_version}'
            params = {
                'url': url,
                'auth': ('', token),
                'verify': verify_certificate,
                'json': b,
            }
            logger.info(f'params: {str(params).replace(token, "***")}')
            response = patch(**params)
            logger.info(f'response.status_code: {response.status_code}')
            logger.info(f'response.text: {response.text}')
            response.raise_for_status()
            ids.append(build_id)
    return ids


def parse_args(args):  # pragma: no cover
    parser = argparse.ArgumentParser(
        description='Cancel builds')
    parser.add_argument(
        'build_id',
        help='The id of the build to cancel')
    parser.add_argument(
        '--token',
        default=environ.get('AZDO_TOKEN', ''),
        help='The Azure DevOps authorisation token')
    parser.add_argument(
        '--organisation',
        default=environ.get('AZDO_ORGANISATION', ''),
        help='The Azure DevOps organisation')
    parser.add_argument(
        '--project',
        default=environ.get('AZDO_PROJECT', ''),
        help='The Azure DevOps project')
    parser.add_argument(
        '-au',
        '--azure_devops_url',
        default='https://dev.azure.com',
        help='The Azure DevOps REST API base url')

    return parser.parse_args(args)


def main(build_id: str, token: str, organisation: str, project: str, azure_devops_url: str) -> None:  # pragma: no cover
    global base_url
    base_url = azure_devops_url

    build_ids = cancel_build(build_id, token, organisation, project)
    print(build_ids)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from re import compile
from sys import argv


def get_split(s):
    for ch in ('_', '-'):
        if ch in s:
            return s.split(ch)
    return compile('[A-Z]?[^A-Z]+').findall(s)


def get_cases(s):
    split = get_split(s)
    capital = [w.capitalize() for w in split]
    lower = [w.lower() for w in split]
    upper = [w.upper() for w in split]
    return [
        ''.join([lower[0]] + capital[1:]),
        ''.join(capital),
        ''.join(lower),
        ''.join(upper),
        '_'.join(lower),
        '_'.join(upper),
        '-'.join(lower),
        '-'.join(upper),
    ]


def get_zipped_cases(strings):
    return zip(*[get_cases(s) for s in strings])


def parse_args(args):
    parser = ArgumentParser(
        description='Prints various string representation of provided strings')
    parser.add_argument('strings', nargs='+')
    return parser.parse_args(args)


def main(strings):
    for items in get_zipped_cases(strings):
        print(' '.join(items))


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from os import name, system
from sys import argv

from vang.bitbucket.api import call


def create_repo(project, repo):
    uri = f'/rest/api/1.0/projects/{project}/repos'
    # request_data = f'{{"name":"{repo}","scmId":"git","forkable":true}}'
    request_data = {'name': repo, 'scmId': 'git', 'forkable': True}
    return call(uri, request_data, 'POST')


def main(project, repository):
    response = create_repo(project, repository)
    commands = '    git remote add origin ' \
        f'{response["links"]["clone"][0]["href"]}\n' \
               '    git push -u origin develop'
    print('If you already have code ready to be pushed to this repository '
          'then run this in your terminal.')
    print(commands)
    if name == 'posix':
        system(f'echo "{commands}\\c" | pbcopy')
        print('(The commands are copied to the clipboard)')


def parse_args(args):
    parser = argparse.ArgumentParser(description='Create Bitbucket repository')
    parser.add_argument('project', help='Project key')
    parser.add_argument('repository', help='Repository')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import logging
from datetime import datetime
from re import fullmatch

from vang.bitbucket.api import get_all
from vang.bitbucket.get_projects import get_projects
from vang.bitbucket.get_repos import get_repos
from vang.core.core import pmap_unordered, get_in
from vang.jenkins.get_jobs import get_jobs

logging.basicConfig(level=logging.INFO)


def matches(v, pattern):
    return fullmatch(pattern, v)


def young_enough(commit_datetime, max_head_commit_age):
    return (datetime.now() - commit_datetime).days < max_head_commit_age


def complement_and_filter_repos(repos, branch_pattern, max_head_commit_age):
    def inner(r):
        branches = {}
        for b in get_all(f'/rest/api/1.0/projects/{r["key"]}/repos/{r["name"]}/branches'):
            branch_name = b['displayId']
            if matches(branch_name, branch_pattern):
                commits = get_all(f'/rest/api/1.0/projects/{r["key"]}/repos/{r["name"]}/commits',
                                  {'until': branch_name}, 1)
                if commits:
                    commit_datetime = datetime.fromtimestamp(commits[0]['authorTimestamp'] / 1000)
                    if young_enough(commit_datetime, max_head_commit_age):
                        branches[branch_name] = commit_datetime

        if branches:
            r = dict(r, branches=branches)
            logging.debug('Including repo: %s', r)
            return r

        logging.debug('Excluding repo: %s', r)
        return None

    for repo in pmap_unordered(
            inner,
            repos,
            processes=30):
        if repo:
            yield repo


def find_repos(project_pattern, repo_pattern):
    for p in get_projects():
        key = p['key']
        if matches(key, project_pattern):
            for r in get_repos(p['key']):
                if matches(r['name'], repo_pattern):
                    repo = {
                        'key': key,
                        'name': r['name'],
                        'cloneUrl': get_in(r, ['links', 'clone', 0, 'href'])
                        # 'cloneUrl': r['links']['clone'][0]['href'],
                    }
                    logging.debug('Found repo: %s', repo)
                    yield repo


def get_repo_info(project_pattern, repo_pattern, branch_pattern, tag_pattern, max_head_commit_age, **kwargs):
    # TODO: Add tag info
    repos = find_repos(project_pattern, repo_pattern)

    updated_repos = {}
    for repo in complement_and_filter_repos(repos, branch_pattern, max_head_commit_age):
        for branch in repo['branches'].keys():
            updated_repos[(repo['key'], repo['name'], branch)] = repo
    return updated_repos


def get_job_info(project_pattern, repo_pattern, branch_pattern, tag_pattern, **kwargs):
    # TODO: Add tag info
    jobs = {}
    for job in get_jobs():
        job_name = job['name']
        try:
            key, name, branch = job_name.split('_')  # Is branch sometimes also tag?
            if matches(key, project_pattern) and matches(name, repo_pattern) and matches(branch, branch_pattern):
                job = {
                    'name': job_name,
                    'url': job['url'],
                    'repo': (key, name, branch)
                }
                jobs[(key, name, branch)] = job
                logging.debug('Including job: %s', job)
            else:
                logging.debug('Excluding job: %s', job_name)
        except ValueError:
            logging.debug('Excluding job: %s', job_name)
    return jobs


def synchronize(cfg):
    repos = get_repo_info(**cfg)
    jobs = get_job_info(**cfg)
    for job in jobs.values():
        if job['repo'] in repos:
            logging.info('Updating job: %s', job)
        else:
            logging.info('Deleting job: %s', job)

    for k, repo in repos.items():
        if k not in jobs:
            logging.info('Creating job: %s', repo)


if __name__ == '__main__':
    from timeit import default_timer as timer

    start = timer()

    cfg = {
        'project_pattern': 'ESAUTO',
        'repo_pattern': '.*',
        'branch_pattern': 'develop|release.*',
        'tag_pattern': [],
        'max_head_commit_age': 30
    }

    synchronize(cfg)

    from timeit import default_timer as timer

    end = timer()
    print(end - start)



#!/usr/bin/env python3
import argparse
import re
from sys import argv

from vang.bitbucket.get_clone_urls import get_clone_urls
from vang.bitbucket.get_projects import get_projects


def get_clone_urls_grep(patterns, command=False):
    return get_clone_urls([
        p['key'] for p in get_projects()
        if any((re.match(r'{}'.format(pattern), p['key'])
                for pattern in patterns))
    ], command)


def main(patterns, command):
    for clone_dir, project, repo, clone_url in get_clone_urls_grep(
            patterns, command):
        print(clone_url)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Get clone urls for project patterns from Bitbucket')
    parser.add_argument('patterns', nargs='+', help='The patterns')
    parser.add_argument(
        '-c', '--command', help='Print as clone commands', action='store_true')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import re
from os import environ
from sys import argv

from requests import get, post


# Found no API for listing views so scraping the web page...
def list_views(url, username, password, verify_certificate):
    response = get(url=url,
                   auth=(username, password),
                   verify=verify_certificate)
    return {r for r in re.findall(r'href="/view/([^/]+)/', response.text, re.MULTILINE)}


def get_view_xml(name, job_names=tuple()):
    job_names_xml = ''.join([f'<string>{n}</string>' for n in job_names])
    return f'''<hudson.model.ListView>
        <name>{name}</name>
        <filterExecutors>false</filterExecutors>
        <filterQueue>false</filterQueue>
        <properties class="hudson.model.View$PropertyList"/>
        <jobNames>{job_names_xml}</jobNames>
        <jobFilters/>
        <columns>
            <hudson.views.StatusColumn/>
            <hudson.views.WeatherColumn/>
            <hudson.views.JobColumn/>
            <hudson.views.LastSuccessColumn/>
            <hudson.views.LastFailureColumn/>
            <hudson.views.LastDurationColumn/>
            <hudson.views.BuildButtonColumn/>
        </columns>
        <recurse>false</recurse>
    </hudson.model.ListView>'''


def get_view(name, url, username, password, verify_certificate):
    uri = f'/view/{name}/config.xml'
    return get(url=f'{url}{uri}',
               auth=(username, password),
               verify=verify_certificate)


def view_exists(name, url, username, password, verify_certificate):
    return get_view(name, url, username, password, verify_certificate).status_code == 200


def do_post(uri, url, username, password, verify_certificate, data=None):
    return post(url=f'{url}{uri}',
                data=data if data else None,
                headers={'Content-Type': 'text/xml'},
                auth=(username, password),
                verify=verify_certificate)


def create_view(name, job_names, url, username, password, verify_certificate):
    uri = f'/createView?name={name}'
    return do_post(uri, url, username, password, verify_certificate, get_view_xml(name, job_names)).status_code


def delete_view(name, url, username, password, verify_certificate):
    uri = f'/view/{name}/doDelete'
    return do_post(uri, url, username, password, verify_certificate).status_code


def update_view(name, job_names, url, username, password, verify_certificate):
    uri = f'/view/{name}/config.xml'
    return do_post(uri, url, username, password, verify_certificate, get_view_xml(name, job_names)).status_code


def set_view(name, job_names, url, username, password, verify_certificate):
    params = (url, username, password, verify_certificate)
    if view_exists(name, *params):
        return update_view(name, job_names, *params)
    return create_view(name, job_names, *params)


def parse_args(args):
    parser = argparse.ArgumentParser(description='Set (Create/Update Jenkins view with jobs')
    parser.add_argument(
        'name',
        help='Jenkins view name',
    )
    parser.add_argument(
        'job_names',
        nargs='+',
        help='Jenkins job names',
    )
    parser.add_argument(
        '--url',
        '-u',
        help='Jenkins url',
        default=environ.get('JENKINS_REST_URL', None)
    )
    parser.add_argument(
        '--username',
        '-n',
        help='Jenkins username',
        default=environ.get('JENKINS_USERNAME', None)
    ),
    parser.add_argument(
        '--password',
        '-p',
        help='Jenkins password',
        default=environ.get('JENKINS_PASSWORD', None)
    )
    parser.add_argument(
        '--verify_certificate',
        '-v',
        help='Verify Jenkins certificate',
        default=not environ.get('JENKINS_IGNORE_CERTIFICATE', None)
    )
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    response_code = set_view(**parse_args(argv[1:]).__dict__)
    print(response_code)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.bitbucket.get_branches import get_branches
from vang.bitbucket.utils import get_repo_specs


def has_branch(repo_specs, branch):
    for spec in repo_specs:
        branches = [
            b['displayId'] for spec, bs in get_branches((spec, ), branch)
            for b in bs
        ]
        yield spec, branch in branches


def main(branch,
         only_has=True,
         only_not_has=False,
         dirs=None,
         repos=None,
         projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, has in has_branch(specs, branch):
        if only_has:
            if has:
                print(f'{spec[0]}/{spec[1]}')
        elif only_not_has:
            if not has:
                print(f'{spec[0]}/{spec[1]}')
        else:
            print(f'{spec[0]}/{spec[1]}, {branch}: {has}')


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Check repository branches in Bitbucket')
    parser.add_argument('branch', help='The branch to check')
    filter_group = parser.add_mutually_exclusive_group()
    filter_group.add_argument(
        '-o',
        '--only_has',
        action='store_true',
        help='Print only repos that has the branch.')
    filter_group.add_argument(
        '-n',
        '--only_not_has',
        action='store_true',
        help='Print only repos that not has the branch.')
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-d',
        '--dirs',
        nargs='*',
        default=['.'],
        help='Git directories to extract repo information from')
    group.add_argument(
        '-r', '--repos', nargs='*', help='Repos, e.g. key1/repo1 key2/repo2')
    group.add_argument(
        '-p', '--projects', nargs='*', help='Projects, e.g. key1 key2')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from os import environ
from sys import argv

from vang.jenkins.api import call

FAILURE = 'FAILURE'
SUCCESS = 'SUCCESS'
NOT_BUILT = 'NOT_BUILT'
UNKNOWN = 'UNKNOWN'


def map_color(color):
    if color == 'blue':
        return SUCCESS
    if color == 'red':
        return FAILURE
    if color == 'notbuilt':
        return NOT_BUILT
    return UNKNOWN


def get_jobs(statuses=(FAILURE, SUCCESS, NOT_BUILT, UNKNOWN), only_names=False,
             url=environ.get('JENKINS_REST_URL', None),
             username=environ.get('JENKINS_USERNAME', None),
             password=environ.get('JENKINS_PASSWORD', None),
             verify_certificate=not environ.get('JENKINS_IGNORE_CERTIFICATE', None)):
    jobs = call('/api/json',
                rest_url=url,
                username=username,
                password=password,
                verify_certificate=verify_certificate)['jobs']
    return [
        job['name'] if only_names else job for job in jobs
        if map_color(job.get('color', None)) in statuses
    ]


def parse_args(args):
    parser = argparse.ArgumentParser(description='Get Jenkins jobs')
    parser.add_argument(
        '-n', '--only_names', action='store_true', help='Get only job names')
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        '-f',
        '--only_failures',
        action='store_true',
        help='Only failure jobs',
    )
    group.add_argument(
        '-s',
        '--only_successes',
        action='store_true',
        help='Only success jobs',
    )
    group.add_argument(
        '-b',
        '--only_not_built',
        action='store_true',
        help='Only not built jobs',
    )
    group.add_argument(
        '-u',
        '--only_unknown',
        action='store_true',
        help='Only unknown status jobs',
    )
    return parser.parse_args(args)


def main(only_failures, only_successes, only_names, only_not_built, only_unknown):
    if only_failures:
        job_statuses = [FAILURE]
    elif only_successes:
        job_statuses = [SUCCESS]
    elif only_not_built:
        job_statuses = [NOT_BUILT]
    elif only_unknown:
        job_statuses = [UNKNOWN]
    else:
        job_statuses = [FAILURE, SUCCESS, NOT_BUILT, UNKNOWN]

    for a_job in get_jobs(job_statuses, only_names):
        print(a_job)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



from json import loads


def get_definition_name(project, repo, branch):
    return f'{project}_{repo}_{branch}'


def get_definition(template, replacements={}, additions={}):
    with open(template, 'rt', encoding='utf-8') as template_file:
        request = template_file.read()
        for key, value in replacements.items():
            request = request.replace(f'##{key}##', value)
        return {**loads(request), **additions}



from functools import reduce


def blank(s):
    return isinstance(s, str) and not s.strip()


def get_in(seq, keys):
    return reduce(lambda mem, k: mem[k], keys, seq) or None


v = {'a': [{'b': 2}]}

print(get_in(v, ('a', 0, 'b')))



#!/usr/bin/env python3

from argparse import ArgumentParser
from os import makedirs
from re import match

from vang.pio.shell import run_command


def create_patch(repo, since_tag, tag, output):
    patch_dir = f'{output}/{tag}'
    makedirs(patch_dir)
    cmd = ' '.join([
        'git log -p --reverse --pretty=email --full-index --binary',
        '--stat',
        '-m',
        '--first-parent'
        f'{since_tag + ".." if since_tag else ""}{tag}',
        f'> {patch_dir}/tag.patch'
    ])
    print(cmd)
    rc, out = run_command(cmd, True, repo)
    return tag, patch_dir, out


def apply_patch(repo, tag, patch_dir):
    print('Apply patch', tag, patch_dir)
    return [
        run_command(cmd, True, repo) for cmd in [
            f'git apply {patch_dir}/*',
            'git add --all',
            f'git commit -m {tag}',
            f'git tag -a {tag} -m {tag}',
        ]
    ]


def get_tags(repo, tag_pattern):
    return [
        tag for tag in run_command('git tag', True, repo)[1].split('\n')
        if match(r'{}'.format(tag_pattern), tag)
    ]


def get_unpatched_tags(patchs_tags, applied_tags):
    return [p for p in patchs_tags if p not in applied_tags]


def is_valid(patchs_tags, applied_tags):
    return all([p == a for p, a in zip(patchs_tags, applied_tags)
                ]) and not len(applied_tags) > len(patchs_tags)


def main(patch_repo, tag_pattern, output, apply_repo):
    patchs_tags = get_tags(patch_repo, tag_pattern)
    applied_tags = get_tags(apply_repo, tag_pattern)
    if is_valid(patchs_tags, applied_tags):
        unpatched_tags = get_unpatched_tags(patchs_tags, applied_tags)
        patch_ranges = zip([None] + unpatched_tags, unpatched_tags)
        created_patches = [
            create_patch(patch_repo, since_tag, until_tag, output)
            for since_tag, until_tag in patch_ranges
        ]
        applied_patches = [
            apply_patch(apply_repo, tag, patch_dir)
            for tag, patch_dir, out in created_patches
        ]
        return applied_patches
    else:
        raise ValueError('Tags are not valid.')


def parse_args(args):
    parser = ArgumentParser(description='Create patches of tags and applies, ' +
                                        'commits and tags them in another repo.')
    parser.add_argument('tag_pattern', help='A tag pattern.')
    parser.add_argument('apply_repo', help='The repo to apply patches to.')
    parser.add_argument(
        '-p',
        '--patch_repo',
        help='The repo to patch from.',
        default='.',
    )
    parser.add_argument(
        '-o',
        '--output',
        help='A directory to put patches in.',
        default='./patch',
    )
    return parser.parse_args(args)



#!/usr/bin/env python3
import argparse
from filecmp import cmp
from os import makedirs, remove, walk
from os.path import exists
from shutil import copyfile, rmtree
from sys import argv


def add(dest_dir, ignore_sub_paths, source_dir):
    adds = []
    for path, dirs, files in walk(source_dir):
        sub_path = path[len(source_dir):]
        if not any([sub_path.startswith(ignore)
                    for ignore in ignore_sub_paths]):
            dest_path = dest_dir + sub_path
            makedirs(dest_path, exist_ok=True)
            for file in files:
                source_file = f'{path}/{file}'
                dest_file = f'{dest_path}/{file}'
                if exists(dest_file):
                    if not cmp(source_file, dest_file):
                        copyfile(source_file, dest_file)
                        adds.append(
                            f'Updated from {source_file} to {dest_file}')
                        print(adds[-1])
                else:
                    copyfile(source_file, dest_file)
                    adds.append(f'Added from {source_file} to {dest_file}')
                    print(adds[-1])
    return adds


def delete(dest_dir, ignore_sub_paths, source_dir):
    deletes = []
    for path, dirs, files in walk(dest_dir):
        sub_path = path[len(dest_dir):]
        if not any([sub_path.startswith(ignore)
                    for ignore in ignore_sub_paths]):
            source_path = source_dir + sub_path
            if not exists(source_path):
                rmtree(path)
                deletes.append(f'Removed dir {path}')
                print(deletes[-1])
            else:
                for file in files:
                    source_file = f'{source_path}/{file}'
                    dest_file = f'{path}/{file}'
                    if not exists(source_file):
                        remove(dest_file)
                        deletes.append(f'Removed file {dest_file}')
                        print(deletes[-1])
    return deletes


def synchronize_dirs(source_dir, dest_dir, ignore_sub_paths=('/.git', )):
    """
    Makes dest_dir look exactly as as source_dir
    with exception of ignore_sub_paths
    :param source_dir: path as string
    :param dest_dir: path as string
    :param ignore_sub_paths: ignored sub paths as list of string
    :return: List describing synchronizations done
    """
    adds = add(dest_dir, ignore_sub_paths, source_dir)
    deletes = delete(dest_dir, ignore_sub_paths, source_dir)
    return adds + deletes


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Makes dest_dir look exactly as as source_dir '
        'with exception of ignore_sub_paths')
    parser.add_argument('source_dir', help='path as string')
    parser.add_argument('dest_dir', help='path as string')
    parser.add_argument(
        '-i',
        '--ignore_sub_paths',
        nargs='+',
        help='ignored sub paths as list of string',
        default=['/.git'])
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    synchronize_dirs(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from os import environ

from vang.jenkins.copy_jobs import copy_jobs

# This script can be used to copy jobs between Jenkins instances, possibly with modifications.
# With a minor modification, it can also be used to update jobs on a Jenkins instance.

# Can be an iterable with pairs of old and new, or a function
# that takes the from_job configuration as parameter and returns the updated configuration.
# Will be used if no job specific replacement has been specified.
replacements = [
    ('old_foo', 'new_foo'),
    ('old_bar', 'new_bar'),
]

# An iterable with strings and/or
# a pair of strings with job names and/or
# a pair of string with job names and job specific replacements (see replacements).
#
# If only a string, the name of from_job and to_job will be the same.
# If a pair of strings, the first element is the name of the from_job and the second will be the name of the clone_job,
# i.e. this can be used to rename the clone.
# A third element can be used to specify job specific replacements, i.e. it overrides the default replacements.
jobs = [
    ('from_job_1', 'to_job_1'),
    ('from_job_2', 'to_job_2', lambda x: x),
    ('from_job_3', 'to_job_3', [('old_baz', 'new_baz')])
]

# Spec of the Jenkins instance to copy from
from_jenkins_spec = {
    'url': environ.get('JENKINS_REST_URL', None),
    'username': environ['JENKINS_USERNAME'],
    'password': environ['JENKINS_PASSWORD'],
    'verify_certificate': not environ.get('JENKINS_IGNORE_CERTIFICATE', None),
}

# Spec of the Jenkins instance to copy to
to_jenkins_spec = dict(from_jenkins_spec, url='<to_jenkins_url>')

# Call the copy_jobs function to perform the actual replacement
copy_jobs(jobs, from_jenkins_spec, to_jenkins_spec, replacements)



#!/usr/bin/env python3

import argparse
from pprint import pprint
from re import search
from sys import argv


def get_dependants(dependency_tree_file, dependency):
    dependency_dict = {}
    with open(dependency_tree_file, 'rt') as file:
        previous_line = ''
        artifact = None
        for line in file.readlines():
            if previous_line.startswith('[INFO] --- maven-dependency-plugin'):
                artifact = line.split(' ')[1].strip()
            elif line.strip() == '[INFO]':
                artifact = None
            elif artifact and search(r'{}'.format(dependency), line):
                key = line.rsplit(' ', maxsplit=1)[1].strip()
                dependants = dependency_dict.get(key, {'direct': set(), 'transitive': set()})
                dependency_dict[key] = dependants
                dependants['transitive' if '|' in line else 'direct'].add(artifact)
            previous_line = line
    return dependency_dict


def main(dependency_tree_file, dependency, only_direct, only_transitive):
    dependants = get_dependants(dependency_tree_file, dependency)
    key = 'direct' if only_direct else 'transitive' if only_transitive else None

    if key:
        d = {}
        for k, v in dependants.items():
            d[k] = v[key]
        dependants = d

    pprint(dependants)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Get direct and transitive dependants from provided dependency_tree_file, '
                    'e.g. ./get_dependants.py ./dependency_tree.txt org.slf4j:slf4j-api')
    parser.add_argument('dependency_tree_file',
                        help='Dependency tree file (created with e.g. mvn dependency:tree > dependency_tree.txt')
    parser.add_argument('dependency', help='The dependency to analyse, e.g. logback. '
                        'Can be a fully qualified dependency, e.g. org.slf4j:slf4j-api:jar:1.7.7:compile, '
                        'or a substring of a dependency, e.g. slf4j-api:jar:1.7.7, '
                        'or a quoted regular expression substring, e.g. "slf4j.*1.7.7".')
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        '-d', '--only_direct', help='Print only direct dependants', action='store_true')
    group.add_argument(
        '-t', '--only_transitive', help='Print only transitive dependants', action='store_true')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from argparse import ArgumentParser
from os import environ
from pprint import pprint
from sys import argv

from requests import get


def get_repositories(url, username, password):
    url = f'{url}/service/rest/v1/repositories'
    response = get(url, auth=(username, password))
    if response.status_code != 200:
        raise OSError(f'{response.status_code}, {response.content}')
    return response.json()


def parse_args(args):
    parser = ArgumentParser(description='Get repositories')
    parser.add_argument('-l', '--url', default=environ.get('NEXUS3_REST_URL', None),
                        help='Nexus3 url, e.g. http://nexus_host:8080')
    parser.add_argument('-u', '--username', default=environ.get('NEXUS3_USERNAME', None), help='Nexus3 username')
    parser.add_argument('-p', '--password', default=environ.get('NEXUS3_PASSWORD', None), help='Nexus3 password')
    return parser.parse_args(args)


def main(url, username, password):
    pprint(get_repositories(url, username, password))


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from argparse import ArgumentParser
from re import fullmatch
from sys import argv
from zipfile import ZipFile


def is_included(patterns, name):
    return any([fullmatch(r'{}'.format(p), name) for p in patterns])


def zip_read(zip_file, patterns, encoding='utf-8'):  # pragma: no cover
    with ZipFile(zip_file, 'r') as z:
        return [(name, z.read(name).decode(encoding)) for name in z.namelist()
                if is_included(patterns, name)]


def main(zip_file, patterns, only_content=False, encoding='utf-8'):
    for file_name, content in zip_read(zip_file, patterns, encoding):
        print('#' * 80)
        if not only_content:
            print('#' * 5, file_name, '#' * 5)
        print(content)


def parse_args(args):
    parser = ArgumentParser(
        description='Read content from files in zip (jar etc)')
    parser.add_argument(
        'zip_file',
        help='The zip file',
    )
    parser.add_argument(
        '-p',
        '--patterns',
        nargs='+',
        default=['.*'],
        help='File name patterns',
    )
    parser.add_argument(
        '-o',
        '--only_content',
        action='store_true',
        help='Print only content',
    )
    parser.add_argument(
        '-e',
        '--encoding',
        default='utf-8',
        help='File encoding',
    )

    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from sys import argv

from vang.tfs.api import call
from vang.tfs.get_repos import get_repos


def get_repo_branches(organisation, project, repository,
                      ref_filter='refs/heads'):
    f = f'/{ref_filter}' if ref_filter else ''
    uri = f'/{organisation}/{project}/_apis/git/repositories' \
          f'/{repository}{f}?includeStatuses=true&api-version=3.2'
    return call(uri)['value']


def get_branches(organisations=None,
                 projects=None,
                 repos=None,
                 names=False):
    if organisations:
        repos = get_repos(organisations=organisations, repo_specs=True)
    if projects:
        repos = get_repos(projects=projects, repo_specs=True)
    if not repos:
        return []

    branch_specs = [(repo, get_repo_branches(*repo.split('/')))
                    for repo in repos]
    if names:
        return [(repo, [branch['name'].split('/')[-1]
                        for branch in branches])
                for repo, branches in branch_specs]
    return [(repo, [dict(branch, name=branch['name'].split('/')[-1])
                    for branch in branches])
            for repo, branches in branch_specs]


def parse_args(args):
    parser = argparse.ArgumentParser(description='Get TFS repos')
    required_group = parser.add_mutually_exclusive_group(required=True)
    required_group.add_argument(
        '-o',
        '--organisations',
        nargs='+',
        help='TFS organisations, e.g organisation')
    required_group.add_argument(
        '-p',
        '--projects',
        nargs='+',
        help='TFS projects, e.g organisation/project')
    required_group.add_argument(
        '-r',
        '--repos',
        nargs='+',
        help='TFS repos, e.g organisation/project/repo')

    parser.add_argument(
        '-n', '--names', action='store_true', help='Get only repo names')
    return parser.parse_args(args)


def main(organisations,
         projects,
         repos,
         names):
    for the_repo, the_branches in get_branches(
            organisations,
            projects,
            repos,
            names,
    ):
        for the_branch in the_branches:
            print(f'{the_repo}: {the_branch}')


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import logging
import random
from argparse import ArgumentParser
from random import choice, shuffle
from string import ascii_lowercase, ascii_uppercase, digits
from sys import argv

logging.basicConfig(level=logging.INFO)


def generate_random_password(special_characters: str = '!@#$%^&*()',
                             number_ascii_lower: int = random.randint(7, 8),
                             number_ascii_upper: int = random.randint(3, 4),
                             number_digits: int = random.randint(3, 4),
                             number_special: int = random.randint(3, 4)) -> str:
    password = [choice(c) for c, n in [[digits, number_digits],
                                       [ascii_lowercase, number_ascii_lower],
                                       [ascii_uppercase, number_ascii_upper],
                                       [special_characters, number_special]]
                for i in range(n)]

    shuffle(password)

    logging.info('Password of length %s: %s lowercase letters, %s uppercase letters, %s digits, %s special characters',
                 len(password),
                 number_ascii_lower,
                 number_ascii_upper,
                 number_digits,
                 number_special)

    return ''.join(password)


def parse_args(args):
    parser = ArgumentParser(description='Generates a password. '
                                        'Default 16-20 characters: '
                                        'lowercase and uppercase letters, digits and special characters.')
    parser.add_argument('-s', '--special_characters',
                        help='Special characters to use in password generation')
    parser.add_argument('-nl', '--number_ascii_lower', type=int,
                        help='Number of ascii lowercase letters in the password')
    parser.add_argument('-nu', '--number_ascii_upper', type=int,
                        help='Number of ascii uppercase letters in the password')
    parser.add_argument('-nd', '--number_digits', type=int,
                        help='Number of digits in the password')
    parser.add_argument('-ns', '--number_special', type=int,
                        help='Number of special characters in the password')
    return parser.parse_args(args)


if __name__ == '__main__':
    args = {k: v for (k, v) in parse_args(argv[1:]).__dict__.items() if v}
    print(generate_random_password(**args))



#!/usr/bin/env python3

import argparse
import traceback
from itertools import count
from os import makedirs
from sys import argv

from vang.pio.shell import run_commands
from vang.tfs.get_projects import get_projects
from vang.tfs.get_repos import get_repos


def clone(commands, root_dir):
    makedirs(root_dir, exist_ok=True)
    yield from run_commands([(cmd, root_dir) for cmd in commands],
                            max_processes=5,
                            timeout=180,
                            check=False)


def get_commands(clone_specs, branch, flat):
    b = f' -b {branch}' if branch else ''
    return [
        f'git clone {c[0]}{b}{"" if flat else f" {c[1]}"}' for c in clone_specs
    ]


def get_clone_specs(projects, flat):
    return [(repo[1]['remoteUrl'],
             repo[1]['name'] if flat else f'{repo[0]}/{repo[1]["name"]}')
            for repo in get_repos(projects=projects)]


def clone_repos(root_dir,
                organisations=None,
                projects=None,
                repos=None,
                branch=None,
                flat=False):
    if organisations:
        projects = get_projects(organisations, project_specs=True)
    elif repos:
        projects = set(['/'.join(r.split('/')[:2]) for r in repos])

    clone_specs = get_clone_specs(projects, flat)

    if repos:
        clone_dirs = [r.split('/')[2] if flat else r for r in repos]
        clone_specs = [(url, clone_dir) for url, clone_dir in clone_specs
                       if clone_dir in clone_dirs]

    commands = get_commands(clone_specs, branch, flat)
    for n, process in zip(count(1), clone(commands, root_dir)):
        try:
            print(str(n).zfill(2), process.stdout.decode(), end='')
        except OSError:  # pragma: no cover
            print(traceback.format_exc())

    return clone_specs


def parse_args(args):
    parser = argparse.ArgumentParser(description='Clone TFS repos')
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        '-o',
        '--organisations',
        nargs='+',
        help='TFS organisations, e.g organisation')
    group.add_argument(
        '-p',
        '--projects',
        nargs='+',
        help='TFS projects, e.g organisation/project')
    group.add_argument(
        '-r',
        '--repos',
        nargs='+',
        help='Repos, e.g. organisation/project/repo')
    parser.add_argument('-b', '--branch', help='The clone branch')
    parser.add_argument(
        '-d', '--clone_dir', default='.', help='The directory to clone into')
    parser.add_argument(
        '-f', '--flat', help='Clone to flat structure', action='store_true')
    return parser.parse_args(args)


def main(clone_dir, organisations, projects, repos, branch, flat):
    for a_repo in clone_repos(
            clone_dir,
            organisations,
            projects,
            repos,
            branch,
            flat,
    ):
        print(a_repo[1])


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from os import makedirs
from sys import argv

from requests import get

from vang.artifactory.list_repo_content import get_repo_content
from vang.core.core import pmap_unordered


def download(output_file, url, username, password):
    with open(output_file, 'wb') as f:
        f.write(get(url, auth=(username, password)).content)
    return url, output_file


def get_path(base_dir, file):
    return f'{base_dir}{file}'


def get_url(artifactory_url, file):
    return f'{artifactory_url}{file}'


def create_dirs(base_dir, files):
    for f in files:
        if f['folder']:
            p = get_path(base_dir, f['uri'])
            makedirs(p, exist_ok=True)


def download_files(artifactory_url, base_dir, files, max_processes=10):
    files_and_urls = [(get_path(base_dir, f['uri']), get_url(artifactory_url, f['uri'])) for f in files if
                      not f['folder']]
    yield from pmap_unordered(lambda uaf: download(uaf[0], uaf[1], '', ''),
                              files_and_urls,
                              processes=max_processes)


def get_artifactory_url(repo_content):
    return repo_content['uri'].replace('api/storage/', '')


def download_repo_content(output_dir, repo_key):
    base_dir = f'{output_dir}/{repo_key}'
    repo_content = get_repo_content(repo_key)
    create_dirs(base_dir, repo_content['files'])
    artifactory_url = get_artifactory_url(repo_content)
    yield from download_files(artifactory_url, base_dir, repo_content['files'])


def main(output_dir, repo_key):
    for url, output_file in download_repo_content(output_dir, repo_key):
        print(url, '->', output_file)


def parse_args(args):
    parser = ArgumentParser(
        description='Download repository content')
    parser.add_argument(
        'output_dir',
        help='Output dir')
    parser.add_argument(
        'repo_key',
        help='Repository key, e.g. libs-release')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from sys import argv

from vang.misc.s import get_zipped_cases
from vang.pio.rsr import get_replace_function, rsr
from vang.pio.shell import run_command
from vang.tfs.clone_repos import clone_repos
from vang.tfs.create_repo import create_repo


def setup(repo, src_branch,  dest_repo, dest_branch, work_dir):
    clone_url, repo_dir = clone_repos(
        work_dir,
        repos=[repo],
        branch=src_branch,
    )[0]

    dest_repo_dir = f'{work_dir}/{dest_repo}'
    run_command(f'mv {repo_dir} {dest_repo_dir}')

    run_command('rm -rf .git', return_output=True, cwd=dest_repo_dir)
    run_command('git init', return_output=True, cwd=dest_repo_dir)
    if not dest_branch == 'master':
        run_command(
            f'git checkout -b {dest_branch}',
            return_output=True,
            cwd=dest_repo_dir,
        )
    return clone_url, dest_repo_dir


def commit_all(repo_dir):
    run_command('git add --all', return_output=True, cwd=repo_dir)
    run_command(
        'git commit -m "Initial commit"',
        return_output=True,
        cwd=repo_dir,
    )


def update(replacements, dest_repo_dir):
    for pair in replacements:
        for old, new in set(get_zipped_cases(pair)):
            rsr(
                old,
                new,
                [dest_repo_dir],
                get_replace_function(False),
            )


def create_and_push_to_dest_repo(
        branch,
        dest_repo,
        dest_repo_dir,
):
    dest_repo_origin = create_repo(dest_repo)['remoteUrl']
    run_command(
        f'git remote add origin {dest_repo_origin}',
        return_output=True,
        cwd=dest_repo_dir)
    run_command(
        f'git push -u origin {branch}',
        return_output=True,
        cwd=dest_repo_dir,
    )
    return dest_repo_origin


def parse_args(args):
    parser = ArgumentParser(
        description='Create a new repo based on template'
                    ' repo.\nExample: create_from_template PCS1906/foo PCS1906/bar'
                    ' -sb sourcebranch -db destinationbranch -d .')
    parser.add_argument(
        'src_repo',
        help='The repo from which to create the new repo (must exist)'
             ', e.g. organisation/project/repo1',
    )
    parser.add_argument(
        'dest_repo',
        help='The new repo (must not exist), '
             'e.g. organisation/project/repo2',
    )
    parser.add_argument(
        '-sb',
        '--src_branch',
        default='develop',
        help='The branch to use, e.g. develop',
    )
    parser.add_argument(
        '-db',
        '--dest_branch',
        default='develop',
        help='The branch to create, e.g. develop. '
             'It will be set as default branch on created repo',
    )
    parser.add_argument(
        '-d',
        '--work_dir',
        default='.',
        help='The directory to create local repo in',
    )
    parser.add_argument(
        '-r',
        '--replacements',
        default=[],
        nargs='+',
        help='Pairs of replacements, e.i. the first in the pair is a string in the src_repo that will be replaced to the second in the pair in the dest_repo',
    )
    return parser.parse_args(args)


def main(
        src_repo,
        src_branch,
        dest_repo,
        dest_branch,
        work_dir,
        replacements,
):
    if len(replacements) % 2 > 0:
        print("Error: Replacements must be pairs")
        exit(1)

    clone_url, dest_repo_dir = setup(
        src_repo,
        src_branch,
        dest_repo,
        dest_branch,
        work_dir,
    )

    replacement_pairs = list(zip(replacements[0::2], replacements[1::2]))
    update(replacement_pairs, dest_repo_dir)

    commit_all(dest_repo_dir)

    dest_repo_origin = create_and_push_to_dest_repo(
        dest_branch,
        dest_repo,
        dest_repo_dir,
    )
    print('Created', dest_repo_origin)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import sys
from os import environ

from requests import delete, get, post, put


def get_page(uri, params, limit, start):
    query = f"&{'&'.join([f'{k}={v}' for k, v in params.items()])}" if params else ''
    response = call(f'{uri}?limit={limit}&start={start}{query}')
    return response['size'], response['values'], response[
        'isLastPage'], response.get('nextPageStart', -1)


def get_all(uri, params=None, take=sys.maxsize):
    limit = min(take, 1000)
    start = 0
    is_last_page = False

    result = []
    while not is_last_page and len(result) < take:
        size, values, is_last_page, start = get_page(uri, params, limit, start)

        if size:
            result += values
    return result[:take]


def call(
        uri,
        request_data=None,
        method='GET',
        only_response_code=False,
        rest_url=environ.get('BITBUCKET_REST_URL', None),
        username=environ.get('BITBUCKET_USERNAME', None),
        password=environ.get('BITBUCKET_PASSWORD', None),
        verify_certificate=not environ.get('BITBUCKET_IGNORE_CERTIFICATE', None),
):
    """Makes a REST call to Bitbucket rest api 1.0.
    Depends on three environment variables:
    * BITBUCKET_REST_URL, e.g. http://myorg.com/stash
    * BITBUCKET_USERNAME, the bitbucket username
    * BITBUCKET_PASSWORD, the bitbucket password

    Args:
        uri (str): e.g. "/rest/api/1.0/projects/{project}/repos/{repo}/branches?filterText={branch}"
        request_data (dict): the JSON request
        method (str): http method
        only_response_code (bool): default False
        rest_url: default environ.get('BITBUCKET_REST_URL', None)
        username (str): default environ.get('BITBUCKET_USERNAME', None)
        password (str): default environ.get('BITBUCKET_PASSWORD', None)
        verify_certificate: True if https certificate should be verified

    Return:
          the JSON response
    """

    m = {'DELETE': delete,
         'GET': get,
         'POST': post,
         'PUT': put,
         }[method]
    # print(username, password, rest_url, request_data, uri)
    params = {'url': f'{rest_url}{uri}', 'auth': (username, password), 'verify': verify_certificate}
    if request_data:
        params['json'] = request_data

    response = m(**params)
    return response.status_code if only_response_code else response.json() if response.text else response.status_code()



#!/usr/bin/env python3
import argparse
from os.path import normpath
from sys import argv

from vang.bitbucket.get_repos import get_all_repos


def get_clone_urls(keys, command=False, branch=False, flat=False):
    for repo in get_all_repos(keys):
        project, repo, clone_url = repo['project']['key'], repo['slug'], repo[
            'links']['clone'][0]['href']
        if command:
            clone_dir = normpath(
                f'{project}/{repo if flat else repo.replace(".", "/")}')
            branch_spec = f'-b {branch} ' if branch else ''
            clone_command = f'git clone {branch_spec}{clone_url} {clone_dir}'
            yield clone_dir, project, repo, clone_command
        else:
            yield None, project, repo, clone_url


def main(projects, command, branch, flat):
    for clone_dir, project, repo, clone_output in get_clone_urls(
            projects,
            command,
            branch,
            flat,
    ):
        print(clone_output)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Get clone urls for projects from Bitbucket.')
    parser.add_argument('projects', nargs='+', help='The projects.')
    parser.add_argument(
        '-c', '--command', help='Print as clone commands', action='store_true')
    parser.add_argument(
        '-b', '--branch', help='Add -b <branch> to clone commands')
    parser.add_argument(
        '-f',
        '--flat',
        help='Make clone commands clone to flat structure',
        action='store_true')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from sys import argv

from vang.jenkins.api import call


def trigger_builds(names):
    return [(name, call(
        f'/job/{name}/build',
        method='POST',
        only_response_code=True,
    )) for name in names]


def parse_args(args):
    parser = argparse.ArgumentParser(description='Trigger Jenkins jobs')
    parser.add_argument(
        'job_names',
        nargs='+',
        help='Jenkins job names',
    )
    return parser.parse_args(args)


def main(job_names):
    for job_name, response_code in trigger_builds(job_names):
        print(job_name, response_code)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.bitbucket.api import get_all


def get_projects():
    return get_all('/rest/api/1.0/projects')


def main(key=False):
    for project in get_projects():
        if key:
            print(project['key'])
        else:
            print(f'{project["key"]}: {project["name"]}')


def parse_args(args):
    parser = argparse.ArgumentParser(description='Get projects from Bitbucket')
    parser.add_argument(
        '-k', '--key', help='Print only project key', action='store_true')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from sys import argv

from vang.tfs.api import call
from vang.tfs.get_projects import get_projects


def list_build_definitions(organisations=None,
                           projects=None,
                           filter_name=None):
    if organisations:
        projects = get_projects(organisations, project_specs=True)
    if not projects:
        return []
    query = f'&name={filter_name}' if filter_name else ''
    build_definitions = {build_definition['name']: build_definition
                         for project in projects
                         for build_definition in
                         call(f'/{project}/_apis/build/definitions?api-version=3.2{query}')['value']}
    return build_definitions


def main(organisations,
         projects,
         filter_name=None,
         ids=False,
         names=False,
         names_and_ids=False,
         urls=False,
         web_urls=False):
    for name, bd in list_build_definitions(organisations, projects, filter_name).items():
        output = (name, bd)
        if ids:
            output = bd['id']
        if names:
            output = bd['name']
        if names_and_ids:
            output = name, bd['id']
        if urls:
            output = bd['url']
        if web_urls:
            output = bd['_links']['web']['href']

        print(output)


def parse_args(args):
    parser = argparse.ArgumentParser(description='List TFS build definitions')
    required_group = parser.add_mutually_exclusive_group(required=True)
    required_group.add_argument(
        '-o',
        '--organisations',
        nargs='+',
        help='TFS organisations, e.g organisation')
    required_group.add_argument(
        '-p',
        '--projects',
        nargs='+',
        help='TFS projects, e.g organisation/project')

    parser.add_argument(
        '-fn',
        '--filter_name',
        help='Filters to definitions whose names equal this value. Append a * to filter to definitions whose names '
             'start with this value')

    optional_group = parser.add_mutually_exclusive_group(required=False)
    optional_group.add_argument(
        '-i', '--ids', action='store_true',
        help='Get only build definition ids')
    optional_group.add_argument(
        '-n', '--names', action='store_true',
        help='Get only build definition names')
    optional_group.add_argument(
        '-ni', '--names_and_ids', action='store_true',
        help='Get only build definition names and ids (as tuples)')
    optional_group.add_argument(
        '-u', '--urls', action='store_true',
        help='Get only build definition urls')
    optional_group.add_argument(
        '-w', '--web_urls', action='store_true',
        help='Get only build definition web urls')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.bitbucket.get_tags import get_tags
from vang.bitbucket.utils import get_repo_specs


def has_tag(repo_specs, tag):
    for spec in repo_specs:
        tags = [t['displayId'] for spec, t in get_tags((spec, ), tag)]
        yield spec, tag in tags


def main(tag, dirs=None, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, has in has_tag(specs, tag):
        print(f'{spec[0]}/{spec[1]}, {tag}: {has}')


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Check repository tags in Bitbucket')
    parser.add_argument('tag', help='The tag to check')
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-d',
        '--dirs',
        nargs='*',
        default=['.'],
        help='Git directories to extract repo information from')
    group.add_argument(
        '-r', '--repos', nargs='*', help='Repos, e.g. key1/repo1 key2/repo2')
    group.add_argument(
        '-p', '--projects', nargs='*', help='Projects, e.g. key1 key2')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import logging
import re
from glob import glob
from os import makedirs
from os.path import isfile
from pprint import pprint
from shutil import copyfile

logging.basicConfig(level=logging.INFO)


def copy_template_files(templates_src, templates_sink, templates):
    copied_template_files = []
    for template in templates:
        sink_path = f'{templates_sink}{template[len(templates_src):]}'
        makedirs(sink_path.rsplit('/', maxsplit=1)[0], exist_ok=True)
        copyfile(template, sink_path)
        copied_template_files.append((template, sink_path))
    return copied_template_files


def map_to_property_file(templates_src, placeholder):
    property_files = []
    property_file = placeholder[2:-1].rsplit('.', maxsplit=1)[0]
    valid_endings = ['-env.properties', '.properties']
    for ending in valid_endings:
        src_path = f'{templates_src}/{property_file}{ending}'
        if isfile(src_path):
            property_files.append(src_path)
    if not property_files:
        logging.critical('No template found for placeholder: %s', placeholder)
    return property_files


def find_templates_in_file(templates_src, file_path, encoding='utf-8'):
    found_templates_in_file = set()
    try:
        with open(file_path, 'rt', encoding=encoding) as f:
            for line in f.readlines():
                m = re.findall(r'@{[\-_.a-zA-Z0-9/]+?}', line)
                if m:
                    for r in m:
                        property_files = map_to_property_file(templates_src, r)
                        if property_files:
                            for property_file in property_files:
                                found_templates_in_file.add(property_file)
                                found_templates_in_file.update(find_templates_in_file(templates_src, property_file))
    except UnicodeDecodeError as e:
        if encoding == 'utf-8':
            logging.critical('%s is not utf-8 encoded', file_path)
            return find_templates_in_file(templates_src, file_path, 'iso-8859-1')
        else:
            logging.error('Can not read %s with utf-8 or iso-8859-1', file_path)
            print(f'Can not read {file_path}', e)
    return found_templates_in_file


def find_templates(glob_pattern, templates_src, endings=('java', 'groovy', 'xml', 'properties')):
    found_templates = set()
    for file_path in glob(glob_pattern, recursive=True):
        if isfile(file_path) and file_path.rsplit('.', maxsplit=1)[1] in endings:
            found_templates.update(find_templates_in_file(templates_src, file_path))
    return found_templates


the_templates_sink = '/Users/magnus/slask/crap/tmp/src/main/templates'
the_templates_src = '/Users/magnus/slask/crap/configuration.war.es/src/main/templates'
code_glob = '/Users/magnus/slask/crap/CSPES/**/src/main/**/*'

the_templates = find_templates(code_glob, the_templates_src)

copied = copy_template_files(the_templates_src, the_templates_sink, the_templates)
logging.info('Copied %d files: %s', len(copied), copied)



#!/usr/bin/env python3
import argparse
import traceback
from json import load
from os import makedirs
from sys import argv

from vang.bitbucket.get_clone_urls import get_clone_urls
from vang.bitbucket.has_branch import has_branch
from vang.core.core import is_included
from vang.pio.shell import run_commands


def should_be_cloned(project, repo, project_config, has_branch_map):
    return has_branch_map[(project, repo)] and is_included(
        repo,
        project_config['excludes'],
        project_config['includes'],
    )


def clone(commands, root_dir):
    makedirs(root_dir, exist_ok=True)
    yield from run_commands([(cmd, root_dir) for cmd in commands],
                            max_processes=25,
                            timeout=60,
                            check=False)


def get_projects_commands(projects, branch=None, flat=False):
    return ((clone_dir, project, repo, command)
            for clone_dir, project, repo, command in get_clone_urls(
                projects,
                True,
                branch,
                flat,
            ))


def get_repos_commands(repos, branch=None, flat=False):
    projects = {repo.split('/')[0] for repo in repos}

    commands = []
    clone_urls_map = {
        f'{project}/{repo}': (clone_dir, project, repo, command)
        for clone_dir, project, repo, command in get_clone_urls(
            projects,
            True,
            branch,
            flat,
        )
    }
    for repo_spec in repos:
        if repo_spec in clone_urls_map:
            commands.append(clone_urls_map[repo_spec])
        else:
            print(f'Warning! Non existing repo: {repo_spec}')

    return commands


def get_config_commands(config, branch=None, flat=False):
    clone_url_specs = tuple(
        get_clone_urls(config['projects'], True,
                       branch if branch else config['branch'], flat))
    has_branch_specs = has_branch([(p, r) for d, p, r, c in clone_url_specs],
                                  config['branch'])
    has_branch_map = {repo_spec: has for repo_spec, has in has_branch_specs}

    return ((clone_dir, project, repo, command)
            for clone_dir, project, repo, command in clone_url_specs
            if should_be_cloned(project, repo, config['projects'][project],
                                has_branch_map))


def main(root_dir,
         projects=None,
         repos=None,
         config=None,
         branch=None,
         flat=False):
    n = 1
    if projects:
        commands = get_projects_commands(projects, branch, flat)
    elif repos:
        commands = get_repos_commands(repos, branch, flat)
    else:
        with open(config, 'rt', encoding='utf-8') as cfg:
            commands = get_config_commands(load(cfg), branch, flat)

    for process in clone(
        [command for clone_dir, project, repo, command in commands],
            root_dir,
    ):
        try:
            print(str(n).zfill(2), process.stdout.decode(), end='')
            n += 1
        except OSError:  # pragma: no cover
            print(traceback.format_exc())


def parse_args(args):
    parser = argparse.ArgumentParser(description='Clone Bitbucket repos')
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        '-p', '--projects', nargs='+', help='Bitbucket projects, e.g key1 key2')
    group.add_argument(
        '-r', '--repos', nargs='+', help='Repos, e.g. key1/repo1 key2/repo2')
    group.add_argument(
        '-c', '--config', help='Configuration file, see clone_repos.json')
    parser.add_argument(
        '-b',
        '--branch',
        help='The clone branch. Overrides branch in configuration file (-c)')
    parser.add_argument(
        '-d', '--root_dir', default='.', help='The directory to clone into')
    parser.add_argument(
        '-f', '--flat', help='Clone to flat structure', action='store_true')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from sys import argv

from vang.artifactory import utils
from vang.artifactory import api
from vang.maven.pom import get_pom_info


def delete_maven_artifact(repository, pom_dirs):
    for pom_dir in pom_dirs:
        pom_info = get_pom_info(utils.get_pom_path(pom_dir))
        base_uri = utils.get_artifact_base_uri(
            repository,
            pom_info['group_id'],
            pom_info['artifact_id'],
            pom_info['version'],
        )
        yield api.call(base_uri, method='DELETE')


def main(repository, pom_dirs):
    for response in delete_maven_artifact(repository, pom_dirs):
        print(response)


def parse_args(args):
    parser = ArgumentParser(
        description='Delete maven artifact from Artifactory')
    parser.add_argument(
        'repository', help='Artifactory repository, e.g. z-release')
    parser.add_argument(
        '-d',
        '--pom_dirs',
        nargs='*',
        default=['.'],
        help='Maven pom directories to extract artifact information from')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from json import dumps
from sys import argv

from vang.tfs.api import call
from vang.tfs.definition_utils import get_definition, get_definition_name


def get_release_definition(template, project, repo, branch, comment=None):
    return get_definition(template,
                          {
                              'name': get_definition_name(project, repo, branch),
                              'project': project,
                              'branch': branch
                          },
                          {'comment': comment} if comment else {})


def create_release_definition(organisation, project, release_definition):
    return call(
        f'/{organisation}/{project}/_apis/release/definitions?api-version=3.2-preview',
        request_data=release_definition,
        method='POST',
        only_response_code=True
    )


def main(project, repo, branch, template, comment=None):
    organisation, project = project.split('/')
    x = get_release_definition(template, project, repo, branch, comment)
    print(dumps(x, indent=4))
    response = create_release_definition(organisation, project,
                                         get_release_definition(template, project, repo, branch, comment))
    print(response)


def parse_args(args):
    parser = argparse.ArgumentParser(description='Create TFS release definitions')
    parser.add_argument(
        'project',
        help='TFS projects, e.g organisation/project')
    parser.add_argument(
        'repo',
        help='The TFS git repo name, e.g. spam.eggs')
    parser.add_argument(
        'branch',
        help='The TFS git repo branch, e.g. 1.0.x')
    parser.add_argument(
        'template',
        help='The TFS release definition template file, e.g. release_definition_template.json')
    parser.add_argument(
        '-c',
        '--comment',
        help='The comment to add to the release definition, e.g. a commit sha of the release definition template')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

from argparse import ArgumentParser
from glob import glob
from os import environ
from os.path import basename
from sys import argv

from vang.maven.pom import get_pom_info
from vang.nexus3.upload import upload
from vang.nexus3.utils import get_artifact_base_uri, get_pom_path


def get_pom_publish_name(pom_path, artifact_id, version):
    pom_name = pom_path.split('/')[-1]
    return pom_name if pom_name.split(
        '.')[-1] == 'pom' else f'{artifact_id}-{version}.pom'


def get_publish_data(artifact_base_uri, path, name):
    return {
        'file_path': path,
        'repository_path': f'{artifact_base_uri}/{name}'
    }


def publish_maven_artifact(repository, pom_dirs, url, username, password):
    for pom_dir in pom_dirs:
        pom_info = get_pom_info(get_pom_path(pom_dir))
        base_uri = get_artifact_base_uri(pom_info['group_id'],
                                         pom_info['artifact_id'],
                                         pom_info['version'])

        publish_data = [get_publish_data(base_uri, pom_info['pom_path'],
                                         get_pom_publish_name(pom_info['pom_path'],
                                                              pom_info['artifact_id'],
                                                              pom_info['version']))] + \
                       [get_publish_data(base_uri, path, basename(path))
                        for path in
                        glob(f'{pom_dir}/**/*.jar', recursive=True) +
                        glob(f'{pom_dir}/**/*.war', recursive=True)]

        yield [
            upload(pd['file_path'], repository, pd['repository_path'], url, username, password)
            for pd in publish_data
        ]


def main(repository, dirs, url, username, password):
    for response in publish_maven_artifact(repository, dirs, url, username, password):
        print(response)


def parse_args(args):
    parser = ArgumentParser(description='Publish maven artifact to Nexus3')
    parser.add_argument(
        'repository', help='Nexus3 repository, e.g. z-release')
    parser.add_argument(
        '-d',
        '--dirs',
        nargs='*',
        default=['.'],
        help='Maven pom directories to extract artifact information from')
    parser.add_argument('-l', '--url', default=environ.get('NEXUS3_REST_URL', None),
                        help='Nexus3 url, e.g. http://nexus_host:8080')
    parser.add_argument('-u', '--username', default=environ.get('NEXUS3_USERNAME', None), help='Nexus3 username')
    parser.add_argument('-p', '--password', default=environ.get('NEXUS3_PASSWORD', None), help='Nexus3 password')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from itertools import product
from multiprocessing.dummy import Pool
from sys import argv

from vang.bitbucket.api import call
from vang.bitbucket.utils import get_repo_specs


def enable_repo_web_hook(spec, url):
    uri = f'/rest/api/1.0/projects/{spec[0]}/repos/{spec[1]}' + \
          '/settings/hooks/com.atlassian.stash.plugin.stash-web-post-' + \
          'receive-hooks-plugin:postReceiveHook/enabled'
    request_data = {'hook-url-0': url}
    return spec, call(uri, request_data, 'PUT')


def enable_web_hook(repo_specs, url, max_processes=10):
    with Pool(processes=max_processes) as pool:
        return pool.starmap(enable_repo_web_hook, product(repo_specs, [url]))


def main(url, dirs, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, response in enable_web_hook(specs, url):
        print(f'{spec[0]}/{spec[1]}: '
              f'{"enabled" if response["enabled"] else " not enabled"}')


def parse_args(args):
    parser = argparse.ArgumentParser(description='Enables Bitbucket webhooks.')
    parser.add_argument(
        'url',
        help='The url which to send repo info to, e.g. '
             'http://10.20.30.40:8002/wildcat/webhook/')
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-d',
        '--dirs',
        nargs='*',
        default=['.'],
        help='Git directories to extract repo information from.')
    group.add_argument(
        '-r', '--repos', nargs='*', help='Repos, e.g. key1/repo1 key2/repo2')
    group.add_argument(
        '-p', '--projects', nargs='*', help='Projects, e.g. key1 key2')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from sys import argv
from typing import Union
from zipfile import ZipFile

from requests import get


def download(output_file: Union[str, bytes, int], url: str, username: str, password: str) -> None:
    with open(output_file, 'wb') as f:
        f.write(get(url, auth=(username, password)).content)


def extract_zip(zip_file):
    with ZipFile(zip_file, 'r') as z:
        z.extractall()


def main(output_file: Union[str, bytes, int], url: str, username: str, password: str, extract: bool) -> None:
    download(output_file, url, username, password)
    if extract:
        extract_zip(output_file)


def parse_args(args):
    parser = argparse.ArgumentParser(description='Download file (requires: pipenv install requests)')
    parser.add_argument('output_file', help='output file, e.g. ./eggs.zip')
    parser.add_argument('url',
                        help='url to download, e.g. http://localhost:9000/job/CSL_xml.ws_1.0.x/3/execution/node/3/ws/target/surefire-reports/*zip*/surefire-reports.zip')
    parser.add_argument('-u', '--username', help='username for basic auth')
    parser.add_argument('-p', '--password', help='password for basic auth')
    parser.add_argument('-e', '--extract', help='if download zip, use this option to extract', action='store_true')
    return parser.parse_args(args)


if __name__ == '__main__':
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python

import argparse
import sys
from http.server import BaseHTTPRequestHandler, HTTPServer

import requests

hostname = 'en.wikipedia.org'


def merge_two_dicts(x, y):
    z = x.copy()  # start with x's keys and values
    z.update(y)  # modifies z with y's keys and values & returns None
    return z


def set_header():
    headers = {
        'Host': hostname
    }

    return headers


class ProxyHTTPRequestHandler(BaseHTTPRequestHandler):
    protocol_version = 'HTTP/1.0'

    def do_HEAD(self):
        self.do_GET(body=False)

    def do_GET(self, body=True):
        sent = False
        try:

            url = 'https://{}{}'.format(hostname, self.path)
            req_header = self.parse_headers()

            print(req_header)
            print(url)
            resp = requests.get(url, headers=merge_two_dicts(req_header, set_header()), verify=False)
            sent = True

            self.send_response(resp.status_code)
            self.send_resp_headers(resp)
            if body:
                self.wfile.write(resp.content)
            return
        finally:
            self.finish()
            self.connection.close()
            if not sent:
                self.send_error(404, 'error trying to proxy')

    def do_POST(self, body=True):
        sent = False
        try:
            url = 'https://{}{}'.format(hostname, self.path)
            content_len = int(self.headers.getheader('content-length', 0))
            post_body = self.rfile.read(content_len)
            req_header = self.parse_headers()

            resp = requests.post(url, data=post_body, headers=merge_two_dicts(req_header, set_header()), verify=False)
            sent = True

            self.send_response(resp.status_code)
            self.send_resp_headers(resp)
            if body:
                self.wfile.write(resp.content)
            return
        finally:
            self.finish()
            self.connection.close()
            if not sent:
                self.send_error(404, 'error trying to proxy')

    def parse_headers(self):
        req_header = {}
        for line in self.headers:
            line_parts = [o.strip() for o in line.split(':', 1)]
            if len(line_parts) == 2:
                req_header[line_parts[0]] = line_parts[1]
        return req_header

    def send_resp_headers(self, resp):
        respheaders = resp.headers
        print('Response Header')
        for key in respheaders:
            if key not in ['Content-Encoding', 'Transfer-Encoding', 'content-encoding', 'transfer-encoding',
                           'content-length', 'Content-Length']:
                print(key, respheaders[key])
                self.send_header(key, respheaders[key])
        self.send_header('Content-Length', str(len(resp.content)))
        self.end_headers()


def parse_args(argv=sys.argv[1:]):
    parser = argparse.ArgumentParser(description='Proxy HTTP requests')
    parser.add_argument('--port', dest='port', type=int, default=9999,
                        help='serve HTTP requests on specified port (default: random)')
    args = parser.parse_args(argv)
    return args


def main(argv=sys.argv[1:]):
    args = parse_args(argv)
    print('http server is starting on port {}...'.format(args.port))
    server_address = ('127.0.0.1', args.port)
    httpd = HTTPServer(server_address, ProxyHTTPRequestHandler)
    print('http server is running as reverse proxy')
    httpd.serve_forever()


if __name__ == '__main__':
    main()



from pprint import pprint

x = [(f'{m} * {n}', m * n) for m in range(1, 13) for n in range(1, 13)]
# x = [x * 3 for x in range(1, 13) if x % 2 == 0]
print(x)
print('Hello', 'World')




# x = [f'{str(m).rjust(2)} * {str(n).rjust(2)} = {str(m * n).rjust(3)}' for m in range(1, 13) for n in range(1, 13)]
# pprint(x)
#
#
# def do_it():
#     for m in range(1, 13):
#         print(m, end=' ')
#
#
# do_it()



import os

from setuptools import setup


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="vang",
    version="1.2.1",
    author="Magnus Bjuvensjö",
    author_email="bjuvensjo@gmail.com",
    description=("Scripts"),
    license="MIT",
    keywords="python artifactory bitbucket git github maven script stash",
    url="https://github.com/bjuvensjo/scripts",
    packages=['vang.artifactory', 'vang.azdo', 'vang.bitbucket', 'vang.core', 'vang.git', 'vang.github', 'vang.jenkins',
              'vang.maven', 'vang.misc', 'vang.nexus3', 'vang.pio', 'vang.tfs', 'vang.wildcat'],
    long_description=read('README.md'),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=['more-itertools', 'requests', 'pytest']
)



#!/usr/bin/env python3
from argparse import ArgumentParser
from sys import argv

from vang.artifactory import api


def list_repos(repo_type=None, only_key=False, only_url=False):
    result = api.call('/api/repositories')
    if repo_type:
        result = [r for r in result if r['type'] == repo_type]
    if only_key:
        result = [r['key'] for r in result]
    if only_url:
        result = [r['url'] for r in result]
    return result


def main(repo_type, only_key, only_url):
    for repo in list_repos(repo_type, only_key, only_url):
        print(repo)


def parse_args(args):
    parser = ArgumentParser(
        description='List repositories in Artifactory')
    parser.add_argument(
        '-t',
        '--repo_type',
        default=None,
        help='Repository type, e.g. LOCAL, REMOTE, VIRTUAL. Default is all.')
    parser.add_argument('-k', '--only_key', help='Print only repo key', action='store_true')
    parser.add_argument('-u', '--only_url', help='Print only repo url', action='store_true')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import re
from glob import glob
from itertools import chain

from os.path import realpath
from sys import argv


def get_reactor_summary(mvn_log):
    """ Returns reactor summary. """
    start = False
    lines = []
    single_project_result = 'FAILURE'
    single_project = ''

    for line in mvn_log:
        # Multi module project build
        if re.match(r'.*Reactor Summary.*', line):
            start = True
        if start:
            lines.append(line.strip())
        if start and re.match(r'^.INFO. -.*', line):
            break

        # Single project build
        if re.match(r'.*Building.*', line) and not single_project:
            single_project = line.strip().replace(' Building', '')
        if single_project and re.match(r'.*BUILD SUCCESS.*', line):
            single_project_result = 'SUCCESS'
            break

    if lines:
        return lines[2:-1]

    return [f'{single_project} {single_project_result}']


def get_project(line):
    """ Returns project of reactor line. """
    start = re.search(r' ', line).end()
    end = re.search(r' ', line[start:]).start() + start
    return line[start:end]


def get_successes(reactor_summary):
    """ Returns success projects """
    return [get_project(l) for l in reactor_summary if re.search('SUCCESS', l)]


def get_failures(reactor_summary):
    """ Returns failure projects """
    return [
        get_project(l) for l in reactor_summary if not re.search('SUCCESS', l)
    ]


def read_lines(log_file):  # pragma: no cover
    with open(log_file, 'rt', encoding='utf-8') as f:
        return f.readlines()


def get_summary(mvn_log_files, do_print=False):
    summaries = tuple(
        chain.from_iterable([
            get_reactor_summary(read_lines(log_file))
            for log_file in mvn_log_files
        ]))
    result = get_successes(summaries), get_failures(summaries)
    if do_print:
        print_summary(*result)
    return result


def print_summary(successes, failures):
    length = 80
    lines = []

    def banner(text='', fill=' '):
        s = ' ' + text + ' ' if text else text
        l = int(length / 2 + len(s) / 2)
        return s.rjust(l, fill).ljust(length, fill)

    def artifact(the_id, success, fill='.'):
        outcome = 'SUCCESS' if success else 'FAILURE'
        return the_id + outcome.rjust(length - len(the_id), fill)

    for a_text in ('', 'Summary', ''):
        lines.append(banner(a_text, '*'))
    for an_id in sorted(successes):
        lines.append(artifact(an_id, True))
    for an_id in sorted(failures):
        lines.append(artifact(an_id, False))
    for a_text in ('', ):
        lines.append(banner(a_text, '*'))

    print('\n'.join(lines))


def main(roots, log_file_name):
    successes, failures = get_summary([
        realpath(p) for root in roots
        for p in glob(f'{root}/**/{log_file_name}', recursive=True)
    ])
    print_summary(successes, failures)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Prints reactor summary of Maven build log files')
    parser.add_argument(
        '-d',
        '--roots',
        help='Root directories in which to find Maven log files',
        nargs='*',
        default=['.'])
    parser.add_argument(
        '-l',
        '--log_file_name',
        help='Name of Maven build log file(s)',
        default='mvn.log')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from pprint import pprint
from re import fullmatch
from sys import argv

from requests import get

from vang.core.core import pmap_ordered


def get_repo_content(repo_key, url, username, password):
    response = get(f'{url}/api/storage/{repo_key}?list&deep=1&listFolders=1', auth=(username, password))
    if response.status_code == 200:
        return response.json()
    else:
        response.raise_for_status()


def filter_repo_files(repo_files, excludes):
    return [f for f in repo_files if not (f['folder'] or any(fullmatch(e, f['uri']) for e in excludes))]


def create_key(artifact_spec):
    return artifact_spec['uri'] if artifact_spec['folder'] else f'{artifact_spec["uri"]}#{artifact_spec["sha1"]}'


def map_to_map(repo_files):
    return {create_key(s): s for s in repo_files}


def diff(a_repo_files, b_repo_files, excludes=('.*maven-metadata.xml',), only_keys=True):
    a_repo_files_map, b_repo_files_map = list(pmap_ordered(
        lambda repo_files: map_to_map(
            filter_repo_files(
                repo_files,
                excludes
            )
        ),
        (a_repo_files, b_repo_files),
        processes=2
    ))

    a_key_set = set(a_repo_files_map.keys())
    b_key_set = set(b_repo_files_map.keys())
    a_only_keys = a_key_set.difference(b_key_set)
    b_only_keys = b_key_set.difference(a_key_set)

    if only_keys:
        return a_only_keys, b_only_keys
    else:
        return (v for k, v in a_repo_files_map.items()
                if k in a_only_keys), (v for k, v in b_repo_files_map.items()
                                       if k in b_only_keys)


def diff_repos(repo_specs, excludes=('.*maven-metadata.xml',), only_keys=True):
    a_repo_content, b_repo_content = list(pmap_ordered(
        lambda spec: get_repo_content(**spec),
        repo_specs,
        processes=2
    ))
    return diff(a_repo_content['files'], b_repo_content['files'], excludes, only_keys)


def main(
        a_repo_key,
        a_artifactory_url,
        a_artifactory_username,
        a_artifactory_password,
        b_repo_key,
        b_artifactory_url,
        b_artifactory_username,
        b_artifactory_password,
):
    specs = (
        {
            'repo_key': a_repo_key,
            'artifactory_url': a_artifactory_url,
            'username': a_artifactory_username,
            'password': a_artifactory_password,
        },
        {
            'repo_key': b_repo_key,
            'artifactory_url': b_artifactory_url,
            'username': b_artifactory_username,
            'password': b_artifactory_password,
        },
    )
    a_only_keys, b_only_keys = diff_repos(specs)

    if a_only_keys:
        print('### Only in a ###')
        pprint(a_only_keys)
    if b_only_keys:
        print('### Only in b ###')
        pprint(b_only_keys)
    print('Done!')


def parse_args(args):
    parser = ArgumentParser(description='Diff content of repositories')
    parser.add_argument('a_repo_key', help='a repo key')
    parser.add_argument('a_artifactory_url', help='a artifactory url')
    parser.add_argument('a_artifactory_username', help='a artifactory username')
    parser.add_argument('a_artifactory_password', help='a artifactory password')
    parser.add_argument('b_repo_key', help='b repo key')
    parser.add_argument('b_artifactory_url', help='b artifactory url')
    parser.add_argument('b_artifactory_username', help='b artifactory username')
    parser.add_argument('b_artifactory_password', help='b artifactory password')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import re
import shlex
from os.path import abspath
from subprocess import run, CalledProcessError
from sys import argv

from vang.pio.shell import run_command


def get_origin_remote_url(repo_dir, remote):
    command = 'git remote -v'
    try:
        rc, output = run_command(command, True, repo_dir)
        for line in output.split('\n'):
            if remote in line:
                return re.split(r'[\t ]+', line)[1]
    except CalledProcessError as cpe:
        print(cpe)
    return None


def open_remote(repo_dir, remote):
    url = get_origin_remote_url(repo_dir, remote)
    if url:
        print(f'Opening {url}')
        run(shlex.split(f'open {url}'))
    else:
        print(f'Found no url to open in {repo_dir}')


def main(repo_dir, remote):  # pragma: no cover
    open_remote(abspath(repo_dir), remote)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Open remote url in browser')
    parser.add_argument(
        '-d',
        '--repo_dir',
        default='.',
        help='Git directory to extract repo information from')
    parser.add_argument(
        '-r',
        '--remote',
        default='origin',
        help='The remote to open')

    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import shlex
from os import environ
from os.path import abspath
from subprocess import run
from sys import argv

from vang.bitbucket.utils import get_branch, get_project_and_repo


def open_remote(
        git_dir,
        repo,
        project,
        base_url,
):
    if project:
        url = f'{base_url}/projects/{project}'
    elif repo:
        p, r = repo.split('/')
        url = f'{base_url}/projects/{p}/repos/{r}/browse'
    else:
        p, r = get_project_and_repo(git_dir)
        url = ''.join([
            f'{base_url}/projects/{p}/repos/{r}/commits?',
            f'until=refs%2Fheads%2F{get_branch(git_dir)}&merges=include'
        ])
    run(shlex.split(f'open {url}'))


def main(repo_dir, repo, project, baseurl):
    open_remote(abspath(f'{repo_dir}/.git'), repo, project, baseurl)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Open Bitbucket url in browser')
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-d',
        '--repo_dir',
        default='.',
        help='Git directory to extract repo information from')
    group.add_argument('-r', '--repo', help='Repo, e.g. key1/repo1')
    group.add_argument('-p', '--project', help='Project, e.g. key1')
    parser.add_argument('-b', '--baseurl', default=environ.get('BITBUCKET_REST_URL', None), help='')

    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
import hashlib
from itertools import count
from os import walk
from os.path import exists
from re import match
from sys import argv


def sha256sum(filename):
    h = hashlib.sha256()
    b = bytearray(128 * 1024)
    mv = memoryview(b)
    with open(filename, 'rb', buffering=0) as f:
        for n in iter(lambda: f.readinto(mv), 0):
            h.update(mv[:n])
    return h.hexdigest()


def diff(master_dir, compare_dir, ignore):
    """
    Returns files in master dir that are not in compare_dir,
    and file in master dir that differ (sha256) from files in compare_dir

    :param master_dir:
    :param compare_dir:
    :param ignore:
    :return:
    """
    missing_files = []
    diffing_files = []

    for root, dirs, files in walk(master_dir):
        for f in files:
            if not any([match(p, f) for p in ignore]):
                mf = f'{root}/{f}'
                cf = f'{compare_dir}/{root[len(master_dir) + 1:]}/{f}'

                if exists(cf):
                    if sha256sum(mf) != sha256sum(cf):
                        diffing_files.append((mf, cf))
                else:
                    missing_files.append(mf)

    return missing_files, diffing_files


def main(master_dir, compare_dir, ignore, only_files=False):
    mfs, dfs = diff(master_dir, compare_dir, ignore)

    if only_files:
        for mf in mfs:
            print(mf)
        for mf, df in dfs:
            print(mf)
    else:
        print('##### Missing files #####')
        for n, mf in zip(count(), mfs):
            print(n, mf)

        print('##### Diffing files #####')
        for n, df in zip(count(), dfs):
            print(n, df)


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Returns files in master dir that are not '
                    'in compare_dir, and file in master dir that differ '
                    '(sha256) from files in compare_dir')
    parser.add_argument('master_dir', help='master dir')
    parser.add_argument('compare_dir', help='compare dir')
    parser.add_argument(
        '-i',
        '--ignore',
        nargs='*',
        default=(
            r'.*lastUpdated',
            r'_remote.repositories',
            r'.*sha1',
            r'maven-metadata-local.xml',
            r'maven-metadata-local.xml',
        ))
    parser.add_argument(
        '-f',
        '--only_files',
        action='store_true'
    )
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)
